/*******************************************************************************
 * Project.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.namespace.QName;

import org.apache.commons.io.FileUtils;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlOptions;

import com.seatojsf.parsers.CopyConfigurationFiles;
import com.seatojsf.parsers.CopyJavaSourceFiles;
import com.seatojsf.parsers.CopyWebContentFiles;
import com.seatojsf.parsers.WebXmlJSF;
import com.sun.java.xml.ns.javaee.FacesConfigApplicationResourceBundleType;
import com.sun.java.xml.ns.javaee.FacesConfigApplicationType;
import com.sun.java.xml.ns.javaee.FacesConfigDocument;
import com.sun.java.xml.ns.javaee.FacesConfigType;
import com.sun.java.xml.ns.javaee.FullyQualifiedClassType;




public class Project {
	
	private File seaProjectPath;
	private File seaSrcPath;
	private File seaWebContentPath;
	private File outputPath;
	private String projectName;
	private File configDir;
	private File templateDir;
	
	
	FacesConfigDocument facesDoc = null;;
	
	
	public Project(File seaProjectPath, File  seaSrcPath, File seaWebContentPath,File outputPath, String projectName, File configDir, File templateDir){
		this.seaProjectPath = seaProjectPath;
		this.seaSrcPath = seaSrcPath;
		this.seaWebContentPath = seaWebContentPath;
		this.outputPath = outputPath;
		this.projectName = projectName;
		this.configDir = configDir;
		this.templateDir = templateDir;
	}
	
	public void migrate(){
		File seaPrj = seaProjectPath;
		File seaPrjSrc = seaSrcPath;
		File seaPrjWebContent = seaWebContentPath;
		SEAProjectInfo seaPrjInfo = new SEAProjectInfo(seaPrj, seaPrjSrc, seaPrjWebContent);
		JSFProjectInfo jsfPrjInfo = new JSFProjectInfo(projectName, outputPath);
		facesDoc = FacesConfigDocument.Factory.newInstance();
		FacesConfigType facesConfig = facesDoc.addNewFacesConfig();
		FlowMigrator fMigrator = null;
		List<WebBean> webbeans = new ArrayList<WebBean>();
		try{
			Iterator<File> webbeansFiles = FileUtils.iterateFiles(seaPrjSrc, new String[]{"webbean"}, true);
			File strutsFile = null;
			File webbeanFile = null;
			String webbeanName = "";
			while (webbeansFiles.hasNext()){
				webbeanFile = webbeansFiles.next();
				webbeanName = webbeanFile.getName().replaceAll("\\.webbean", "");
				strutsFile = new File(seaPrjWebContent+"/WEB-INF/struts-config-"+webbeanName+".xml");
				webbeans.add(WebBean.parse(webbeanFile,strutsFile));
			}
			fMigrator = new FlowMigrator(webbeans, seaPrjInfo, jsfPrjInfo, facesConfig, templateDir);
			fMigrator.migrate();
			FacesConfigApplicationType app = addApplicationFaces(facesConfig);
			addResourceBundle(seaPrjInfo, app);
			writeWebXml(seaPrjInfo, jsfPrjInfo);
			writeFacesConfigFile(jsfPrjInfo);
			copyAllFiles(seaPrjInfo, jsfPrjInfo);
		}catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	private FacesConfigApplicationType addApplicationFaces(FacesConfigType facesConfig) {
		FacesConfigApplicationType app = facesConfig.addNewApplication();
		return app;
		
	}

	private void addResourceBundle(SEAProjectInfo seaPrjInfo, FacesConfigApplicationType app) {
		MultiLanguageSea ml = new MultiLanguageSea(seaPrjInfo);
		Collection<String> col = ml.getLanguagesFilesNames();
		Iterator<String> it = col.iterator();
		String resource=null;
		boolean first = true;
		String sFirst;
		while (it.hasNext()){
			resource = it.next();
			if (first){
				if (resource.contains("_")){
					StringTokenizer st = new StringTokenizer(resource,"_");
					sFirst = st.nextToken();
				}
				else{
					sFirst = resource;
				}
				FacesConfigApplicationResourceBundleType resourceType = app.addNewResourceBundle();
				FullyQualifiedClassType name = resourceType.addNewBaseName();
				name.setStringValue(sFirst);
				com.sun.java.xml.ns.javaee.String var = resourceType.addNewVar();
				var.setStringValue("msg");
				com.sun.java.xml.ns.javaee.String message = app.addNewMessageBundle();
				message.setStringValue(resource);
			}
			else{
				com.sun.java.xml.ns.javaee.String message = app.addNewMessageBundle();
				message.setStringValue(resource);				
			}
		}
		
	}

	private void writeFacesConfigFile(JSFProjectInfo jsfPrjInfo)
			throws IOException {
		XmlCursor cursor = facesDoc.newCursor();
		if (cursor.toFirstChild())
		{
			cursor.setAttributeText(new QName("http://www.w3.org/2001/XMLSchema-instance","schemaLocation"), "http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-facesconfig_2_0.xsd");
			cursor.toNextAttribute();
			cursor.setAttributeText(new QName("","version"), "2.0");
		}
		cursor.dispose();
		XmlOptions options = new XmlOptions();
		options.setUseDefaultNamespace();
		options.setSavePrettyPrint();
		File f = new File(jsfPrjInfo.getWebcontentPath().getAbsolutePath()+"/WEB-INF/faces-config.xml");
		FileUtils.write(f, "");
		facesDoc.save(f,options);
		
	}

	private void copyAllFiles(SEAProjectInfo seaPrjInfo,
			JSFProjectInfo jsfPrjInfo) {
		CopyJavaSourceFiles copyJavaSource = new CopyJavaSourceFiles(seaPrjInfo, jsfPrjInfo);
		copyJavaSource.copy();
		CopyWebContentFiles copyWebContent = new CopyWebContentFiles(seaPrjInfo, jsfPrjInfo);
		copyWebContent.copy();
		CopyConfigurationFiles copyConf = new CopyConfigurationFiles(configDir, jsfPrjInfo);
		copyConf.copy();
	}

	private void writeWebXml(SEAProjectInfo seaPrjInfo,
			JSFProjectInfo jsfPrjInfo) throws XmlException, IOException,
			Exception {
		WebXmlSEA webXmlSea = WebXmlSEA.parse(new File(seaPrjInfo.getWebcontentPath()+"/WEB-INF/web.xml"));
		WebXmlJSF webXmlJsf =new WebXmlJSF(jsfPrjInfo, webXmlSea);
		webXmlJsf.writeWebXml();
	}
	
	
	
	
	
	

}
