/*******************************************************************************
 * Servlet.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

public class Servlet {

	private String servletName;
	private String servletClass;
	private Collection<InitParam> initParams;
	private Integer loadOnStartup;

	public Servlet() {
		this.servletName = "";
		this.servletClass = "";
		this.initParams = new ArrayList<InitParam>();
		this.loadOnStartup = null;
	}

	public Servlet(String servletName, String servletClass) {
		this.servletName = servletName;
		this.servletClass = servletClass;
		this.initParams = new ArrayList<InitParam>();
	}

	public Servlet(String servletName, String servletClass,
			Integer loadOnStartup) {
		this.servletName = servletName;
		this.servletClass = servletClass;
		this.initParams = new ArrayList<InitParam>();
		this.loadOnStartup = loadOnStartup;
	}

	public String getServletName() {
		return servletName;
	}

	public void setServletName(String servletName) {
		this.servletName = servletName;
	}

	public String getServletClass() {
		return servletClass;
	}

	public void setServletClass(String servletClass) {
		this.servletClass = servletClass;
	}

	public Collection<InitParam> getInitParams() {
		return initParams;
	}

	public void addInitParam(InitParam initParam) {
		this.initParams.add(initParam);
	}

	public String toString() {
		return ReflectionToStringBuilder.toString(this,
				ToStringStyle.MULTI_LINE_STYLE);
	}

	public Integer getLoadOnStartup() {
		return loadOnStartup;
	}

	public void setLoadOnStartup(Integer loadOnStartup) {
		this.loadOnStartup = loadOnStartup;
	}

	@Override
	public boolean equals(Object o) {
		boolean equals = false;
		if (this == o) {
			equals = true;
		}
		if (this.getServletName().equals(((Servlet) o).getServletName())) {
			equals = true;
		}
		return equals;
	}

	@Override
	public int hashCode() {
		return servletName.hashCode();
	}

}
