/*******************************************************************************
 * WebBean.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.apache.xmlbeans.XmlException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.atsistemas.webbeans.model.v1.BeanSection;
import com.atsistemas.webbeans.model.v1.BeanSection.BEAN;
import com.atsistemas.webbeans.model.v1.Flow;
import com.atsistemas.webbeans.model.v1.WEBBEANDocument;
import com.atsistemas.webbeans.model.v1.WebbeanType;


/**
 * 
 * Clase que permite leer un fichero webbean de SEA y extraer su información 
 * de Flow y lista de Beans. Se utiliza la tecnología XMLBeans de apache.
 */
public class WebBean {
	
	private List<BEAN> beans;
	private Flow flow;
	private String id;
	private List<View> views;
	

	private WebBean(){
	}
	
	
	public List<BEAN> getBeans() {
		return beans;
	}


	public Flow getFlow() {
		return flow;
	}
	
	



	public String getId() {
		return id;
	}


	private static WebBean parseWebbeanFile(File webbeanFile)throws IOException, XmlException{
		WEBBEANDocument webbeanDoc = null;
		WebBean wb = new WebBean();
		webbeanDoc = WEBBEANDocument.Factory.parse(webbeanFile);
		WebbeanType webbean = webbeanDoc.getWEBBEAN(); 
		wb.flow = webbean.getFLOW();
		BeanSection model = webbean.getMODEL();
		if (null!=model){
			wb.beans = model.getBEANList();				
		}
		else {
			wb.beans = null;
		}
		wb.id = webbeanFile.getName().replaceFirst("\\.webbean", "");
		return wb;
	}
	
	
	private static List<View> parseStrutsConfigFile(File strutsConfigFile)throws FileNotFoundException, XPathExpressionException{
		ArrayList<View> list = new ArrayList<View>();
		XPathFactory xFactory =  XPathFactory.newInstance();
		XPath xPath = xFactory.newXPath();
		InputSource inputSource = new InputSource(new FileInputStream(strutsConfigFile));
		NodeList nodes = (NodeList) xPath.evaluate("//global-forwards/forward", inputSource, XPathConstants.NODESET);
		Node node = null;;
		String name = "";
		String path = "";
		for (int i=0; i<nodes.getLength(); i++ ){
			node = nodes.item(i);
			name = ((Element)node).getAttribute("name");
			path = ((Element)node).getAttribute("path");
			list.add(new View(path,name));
		}
		
		return list;
	}
	
	public String toString(){
		return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
	}	


	public static WebBean parse(File webbeanFile, File strutsConfigFile) throws IOException, XmlException, FileNotFoundException, XPathExpressionException{
		WebBean b = WebBean.parseWebbeanFile(webbeanFile);
		b.views = WebBean.parseStrutsConfigFile(strutsConfigFile);
		return b;
	}


	public List<View> getViews() {
		return views;
	}
	
	
	

	
	

}
