/*******************************************************************************
 * CommandLine.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.commandline;

import static org.kohsuke.args4j.ExampleMode.ALL;
import java.io.File;
import java.io.IOException;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

import com.seatojsf.appflowmig.Project;

public class CommandLine {

	@Option(name = "-sprojectpath", usage = "Sets a sprojectpath")
	public File sprojectpath=null;

	@Option(name = "-seasrcpath", usage = "Sets a seasrcpath")
	public File seasrcpath=null;

	@Option(name = "-seawebcontentpath", usage = "Sets a seawebcontentpath")
	public File seawebcontentpath=null;
	
	@Option(name = "-output", usage = "Sets a output")
	public File output=null;
	
	@Option(name = "-jsfprojectname", usage = "Sets a jspprojectname")
	public String jsfprojectname=null;
	
	@Option(name = "-configdir", usage = "Sets a config directory")
	public File configdir=null;	
	
	@Option(name = "-templateDir", usage = "Sets a config directory")
	public File templateDir=null;	
	


	//@Argument
	//private List<String> arguments = new ArrayList<String>();

	public static void main(String[] args) throws IOException {
		CommandLine cm = new CommandLine();
		cm.doMain(args);
		Project migrator = new Project(cm.sprojectpath, cm.seasrcpath, cm.seawebcontentpath, cm.output, cm.jsfprojectname, cm.configdir, cm.templateDir);
		migrator.migrate();
		
	}

	public void doMain(String[] args) throws IOException {
		CmdLineParser parser = new CmdLineParser(this);
		parser.setUsageWidth(80);
		try {
			// parse the arguments.
			parser.parseArgument(args);
			//if (arguments.isEmpty())
			//	throw new CmdLineException(parser, "No argument is given");
			if ( (this.output==null) || (this.seasrcpath==null) || 
					(this.seawebcontentpath==null) || (this.sprojectpath==null)  || 
					(this.jsfprojectname==null)  || (this.configdir==null)   
					|| (this.templateDir==null)){
				throw new CmdLineException(parser, "No argument is given");
			}
			

		} catch (CmdLineException e) {
			System.err.println(e.getMessage());
			System.err.println("java com.seatojsf.commandline.CommandLine [options...] arguments...");
			parser.printUsage(System.err);
			System.err.println();
			System.err.println("  Example: java com.seatojsf.commandline.CommandLine"
					+ parser.printExample(ALL));
			return;
		}
		// access non-option arguments
		//System.out.println("other arguments are:");
		//for (String s : arguments)
		//	System.out.println(s);
	}

}
