/*******************************************************************************
 * CopyConfigurationFiles.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;

import org.apache.commons.io.FileUtils;

import com.seatojsf.appflowmig.JSFProjectInfo;

public class CopyConfigurationFiles {
	
	
	private File configDirectory;
	private JSFProjectInfo jsfProjectInfo;
	
	
	public CopyConfigurationFiles(File configDirectory, JSFProjectInfo jsfProjectInfo){
		this.configDirectory = configDirectory;
		this.jsfProjectInfo = jsfProjectInfo;
	}
	
	
	
	public void copy(){
		copyProject();
		copyClasspath();
		copySettings();
		copyLib();
		
	}
	
	void copyLib(){
		try{
			FileUtils.copyDirectory(new File(configDirectory + "/lib"), new File(jsfProjectInfo.getWebcontentPath() + "/WEB-INF/lib"));
		}catch (Exception e) {
			throw new RuntimeException(e);
		}
	}



	void copyClasspath() {
		try{
			FileUtils.copyFile(new File(configDirectory + "/.classpath"), 
					new File(jsfProjectInfo.getOutputDirectory() + "/" + jsfProjectInfo.getProjectName() + "/.classpath"));
		}catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	



	void copyProject() {
		try{
			File project = new File(configDirectory + "/.project");
			String content = FileUtils.readFileToString(project);
			content = content.replaceFirst("@projectName", jsfProjectInfo.getProjectName());
			FileUtils.write(new File(jsfProjectInfo.getOutputDirectory() + "/" + jsfProjectInfo.getProjectName() + "/.project"), content);
		}catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	
	void copySettings() {
		File settings = new File(configDirectory + "/.settings");
		File jsfPrDir = new File(jsfProjectInfo.getOutputDirectory() + "/" + jsfProjectInfo.getProjectName());
		File jsfPrDirSettings = new File(jsfProjectInfo.getOutputDirectory() + "/" + jsfProjectInfo.getProjectName() + "/.settings");
		try{
			FileUtils.copyFile(new File(settings + "/.jsdtscope"),new File( jsfPrDirSettings + "/.jsdtscope"));
			FileUtils.copyFile(new File(settings + "/org.eclipse.jdt.apt.core.prefs"),new File( jsfPrDirSettings + "/org.eclipse.jdt.apt.core.prefs"));
			FileUtils.copyFile(new File(settings + "/org.eclipse.jdt.core.prefs"),new File( jsfPrDirSettings + "/org.eclipse.jdt.core.prefs"));
			FileUtils.copyFile(new File(settings + "/org.eclipse.wst.common.project.facet.core.xml"),new File( jsfPrDirSettings + "/org.eclipse.wst.common.project.facet.core.xml"));
			FileUtils.copyFile(new File(settings + "/org.eclipse.wst.jsdt.ui.superType.container"),new File( jsfPrDirSettings + "/org.eclipse.wst.jsdt.ui.superType.container"));
			FileUtils.copyFile(new File(settings + "/org.eclipse.wst.jsdt.ui.superType.name"),new File( jsfPrDirSettings + "/org.eclipse.wst.jsdt.ui.superType.name"));
			FileUtils.copyFile(new File(settings + "/org.eclipse.wst.common.project.facet.core.prefs.xml"),new File( jsfPrDirSettings + "/org.eclipse.wst.common.project.facet.core.prefs.xml"));
			File commonComponent = new File(configDirectory + "/.settings/org.eclipse.wst.common.component");
			String content = FileUtils.readFileToString(commonComponent);
			content = content.replaceAll("@projectName", jsfProjectInfo.getProjectName());
			FileUtils.write(new File(jsfProjectInfo.getOutputDirectory() + "/" + jsfProjectInfo.getProjectName() + "/.settings/org.eclipse.wst.common.component"), content);			
		}catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	
	
	
	
	

}
