/*******************************************************************************
 * IResources.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

/**
 * 
 * Constantes de aplicación utilizadas para los parsers Java
 * y elementos de flujo xhtml
 */
public interface IResources {
	
	
	
	public static final String S_LINE_SEPARATOR = System.getProperty("line.separator");
	
	
	/**
	 * Plantilla XHTML para las redirecciones y control de flujo
	 */
	public static final String XHTML_TEMPLATE = 
		"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +S_LINE_SEPARATOR+
		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">"+S_LINE_SEPARATOR+
		"<html xmlns=\"http://www.w3.org/1999/xhtml\""+S_LINE_SEPARATOR+
		"xmlns:ui=\"http://java.sun.com/jsf/facelets\""+S_LINE_SEPARATOR+
		"xmlns:f=\"http://java.sun.com/jsf/core\""+S_LINE_SEPARATOR+
		"xmlns:h=\"http://java.sun.com/jsf/html\">"+S_LINE_SEPARATOR+
		"<h:head>"+S_LINE_SEPARATOR+
		"<f:event listener=\"#{@javaManaged}\" type=\"preRenderView\"></f:event>"+S_LINE_SEPARATOR+
		"<meta http-equiv=\"Content-Type\"	content=\"application/xhtml+xml; charset=UTF-8\" />"+S_LINE_SEPARATOR+
		"</h:head>"+S_LINE_SEPARATOR+
		"<h:body>"+S_LINE_SEPARATOR+
		"</h:body>"+S_LINE_SEPARATOR+
		"</html>";
	
	
	/**
	 * Clases que se necesitan para realizar un jsfExecute.
	 */
	public static final String[] JSF_EXECUTE_IMPORTS = new String[]{
		"javax.faces.context.FacesContext",
		"javax.servlet.http.HttpServletRequest",
		"com.seatojsf.api.RedirectorValues", 
		"net.sf.json.JSONObject"};
	
	/**
	 * Método jsfExecute que se inyectará en los commands.
	 */
	public static final String JSF_COMMAND_EXECUTE = 
		"\tpublic void jsfExecute(String redirect)throws Throwable{"+S_LINE_SEPARATOR+
		"\t\tStrutsContext ctx = new StrutsContext();"+S_LINE_SEPARATOR+
		"\t\texecute(ctx);"+S_LINE_SEPARATOR+
		"\t\tFacesContext contex = FacesContext.getCurrentInstance();"+S_LINE_SEPARATOR+
		"\t\tHttpServletRequest request = (HttpServletRequest)contex.getExternalContext().getRequest();"+S_LINE_SEPARATOR+
		"\t\tString context = request.getContextPath();"+S_LINE_SEPARATOR+
		"\t\tJSONObject jsonObject = JSONObject.fromObject(redirect);"+S_LINE_SEPARATOR+  
		"\t\tObject values = JSONObject.toBean(jsonObject,RedirectorValues.class);"+S_LINE_SEPARATOR+
		"\t\tRedirectorValues r = (RedirectorValues)values;"+S_LINE_SEPARATOR+
		"\t\tcontex.getExternalContext().redirect(context+r.getValues().get(\"\"));"+S_LINE_SEPARATOR+
	"\t}"+S_LINE_SEPARATOR+S_LINE_SEPARATOR+S_LINE_SEPARATOR;

	
	/**
	 * Método jsfExecute que se inyectará en los ifs.
	 */
	public static final String JSF_IF_EVALUATE = 
		"\tpublic void jsfExecute(String redirect)throws Throwable{"+S_LINE_SEPARATOR+
		"\t\tStrutsContext ctx = new StrutsContext();"+S_LINE_SEPARATOR+
		"\t\tboolean result = evaluate(ctx);"+S_LINE_SEPARATOR+
		"\t\tFacesContext contex = FacesContext.getCurrentInstance();"+S_LINE_SEPARATOR+
		"\t\tHttpServletRequest request = (HttpServletRequest)contex.getExternalContext().getRequest();"+S_LINE_SEPARATOR+
		"\t\tString context = request.getContextPath();"+S_LINE_SEPARATOR+
		"\t\tJSONObject jsonObject = JSONObject.fromObject(redirect);"+S_LINE_SEPARATOR+  
		"\t\tObject values = JSONObject.toBean(jsonObject,RedirectorValues.class);"+S_LINE_SEPARATOR+
		"\t\tRedirectorValues r = (RedirectorValues)values;"+S_LINE_SEPARATOR+
		"\t\tif (result){"+S_LINE_SEPARATOR+
		"\t\t	contex.getExternalContext().redirect(context+r.getValues().get(\"true\"));"+S_LINE_SEPARATOR+			
		"\t\t}"+S_LINE_SEPARATOR+
		"\t\telse{"+S_LINE_SEPARATOR+
		"\t\t	contex.getExternalContext().redirect(context+r.getValues().get(\"false\"));"+S_LINE_SEPARATOR+			
		"\t\t}"+S_LINE_SEPARATOR+
		"\t}"+S_LINE_SEPARATOR+S_LINE_SEPARATOR+S_LINE_SEPARATOR;
	
	
	/**
	 * Método jsfExecute que se inyectará en los switch.
	 */
	public static final String JSF_SWITCH_EVALUATE =
		"\tpublic void jsfExecute(String redirect)throws Throwable{"+S_LINE_SEPARATOR+
		"\t\tStrutsContext ctx = new StrutsContext();"+S_LINE_SEPARATOR+
		"\t\tString result = evaluate(ctx);"+S_LINE_SEPARATOR+
		"\t\tFacesContext contex = FacesContext.getCurrentInstance();"+S_LINE_SEPARATOR+
		"\t\tHttpServletRequest request = (HttpServletRequest)contex.getExternalContext().getRequest();"+S_LINE_SEPARATOR+
		"\t\tString context = request.getContextPath();"+S_LINE_SEPARATOR+
		"\t\tJSONObject jsonObject = JSONObject.fromObject(redirect);"+S_LINE_SEPARATOR+  
		"\t\tObject values = JSONObject.toBean(jsonObject,RedirectorValues.class);"+S_LINE_SEPARATOR+
		"\t\tRedirectorValues r = (RedirectorValues)values;"+S_LINE_SEPARATOR+
		"\t\tif (!\"\".equals(r.getValues().get(result))){"+S_LINE_SEPARATOR+
		"\t\t	contex.getExternalContext().redirect(context+r.getValues().get(result));"+S_LINE_SEPARATOR+			
		"\t\t}"+S_LINE_SEPARATOR+
		"\t\telse{"+S_LINE_SEPARATOR+
		"\t\t	contex.getExternalContext().redirect(context+r.getValues().get(\"default\"));"+S_LINE_SEPARATOR+
		"\t\t}"+S_LINE_SEPARATOR+
		"\t}"+S_LINE_SEPARATOR+S_LINE_SEPARATOR+S_LINE_SEPARATOR;
	
	
	
	
	public static final String JSF_INVOKE_CLASS =
		"package es.allianz.invoke;"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"import java.util.Stack;"+S_LINE_SEPARATOR+
		"import javax.el.ELContext;"+S_LINE_SEPARATOR+
		"import javax.faces.context.FacesContext;"+S_LINE_SEPARATOR+
		"import javax.servlet.http.HttpServletRequest;"+S_LINE_SEPARATOR+
		"import net.sf.json.JSONObject;"+S_LINE_SEPARATOR+
		"import com.seatojsf.api.RedirectorValues;"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"public class @ClassName {"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"public void jsfExecute(String redirect)throws Throwable{"+S_LINE_SEPARATOR+
		"FacesContext contex = FacesContext.getCurrentInstance();"+S_LINE_SEPARATOR+
		"HttpServletRequest request = (HttpServletRequest)contex.getExternalContext().getRequest();"+S_LINE_SEPARATOR+
		"String context = request.getContextPath();"+S_LINE_SEPARATOR+
		"JSONObject jsonObject = JSONObject.fromObject(redirect);"+S_LINE_SEPARATOR+
		"Object values = JSONObject.toBean(jsonObject,RedirectorValues.class);"+S_LINE_SEPARATOR+
		"RedirectorValues r = (RedirectorValues)values;"+S_LINE_SEPARATOR+
		"ELContext elContext = FacesContext.getCurrentInstance().getELContext();"+S_LINE_SEPARATOR+
		"Stack<String> managementInvocation = (Stack<String>)contex.getApplication().getELResolver().getValue(elContext, null, \"ManagementInvocation\");"+S_LINE_SEPARATOR+
		"managementInvocation.push(context+r.getValues().get(\"gotostate\"));"+S_LINE_SEPARATOR+
		"contex.getExternalContext().redirect(context+r.getValues().get(\"target\"));"+S_LINE_SEPARATOR+		
		"}"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"}";
	
	
	public static final String JSF_RETURN_CLASS = 
		"package es.allianz.ret;"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"import java.util.Stack;"+S_LINE_SEPARATOR+
		"import javax.el.ELContext;"+S_LINE_SEPARATOR+
		"import javax.faces.context.FacesContext;"+S_LINE_SEPARATOR+
		"import javax.servlet.http.HttpServletRequest;"+S_LINE_SEPARATOR+
		"import net.sf.json.JSONObject;"+S_LINE_SEPARATOR+
		"import com.seatojsf.api.RedirectorValues;"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"public class @ClassName {"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"public void jsfExecute(String redirect)throws Throwable{"+S_LINE_SEPARATOR+
		"FacesContext contex = FacesContext.getCurrentInstance();"+S_LINE_SEPARATOR+
		"HttpServletRequest request = (HttpServletRequest)contex.getExternalContext().getRequest();"+S_LINE_SEPARATOR+
		"String context = request.getContextPath();"+S_LINE_SEPARATOR+
		"JSONObject jsonObject = JSONObject.fromObject(redirect);"+S_LINE_SEPARATOR+
		"Object values = JSONObject.toBean(jsonObject,RedirectorValues.class);"+S_LINE_SEPARATOR+
		"RedirectorValues r = (RedirectorValues)values;"+S_LINE_SEPARATOR+
		"ELContext elContext = FacesContext.getCurrentInstance().getELContext();"+S_LINE_SEPARATOR+
		"Stack<String> managementInvocation = (Stack<String>)contex.getApplication().getELResolver().getValue(elContext, null, \"ManagementInvocation\");"+S_LINE_SEPARATOR+
		"String redirec = managementInvocation.pop();"+S_LINE_SEPARATOR+
		"contex.getExternalContext().redirect(redirec);"+S_LINE_SEPARATOR+		
		"}"+S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		S_LINE_SEPARATOR+
		"}";		
	
	
	
	
}
