/*******************************************************************************
 * JavaState.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;

import org.apache.commons.io.FileUtils;

import com.seatojsf.appflowmig.FlowStateNode;
import com.seatojsf.appflowmig.JSFProjectInfo;
import com.seatojsf.appflowmig.SEAProjectInfo;


/**
 * 
 * Clase que parsea las clases java dentro del flujo de SEA y 
 * genera la nueva clase compatible para JSF.
 */
public class JavaState {

	
	private String seaJavaFile;
	private String jsfJavaFile;
	private String type;
	
	private String clazz;
	private FlowStateNode node;
	
	
	public JavaState(FlowStateNode node, SEAProjectInfo seaProjectInfo, JSFProjectInfo jsfProjectInfo){
		this.node = node;
		this.clazz = FlowStateNode.getPackageAndClass(node);
		this.seaJavaFile = seaProjectInfo.concatSrcPathToClass(clazz);
		this.jsfJavaFile = jsfProjectInfo.concatSrcPathToClass(clazz);
		this.type = FlowStateNode.getType(node);
	}
	

	public void generateJSFJavaFile() {
		try {
			if ("return".equals(type)){
				generateFile(IResources.JSF_RETURN_CLASS.replaceFirst("@ClassName", (FlowStateNode.getClassName(node))));						
			}
			else if ("invoke".equals(type)){
				generateFile(IResources.JSF_INVOKE_CLASS.replaceFirst("@ClassName", ( FlowStateNode.getClassName(node) )));
			}
			else{
				String seaFileContent = FileUtils.readFileToString(new File(seaJavaFile));
				seaFileContent = addImports(seaFileContent);
				seaFileContent = addJsfMethod(seaFileContent);
				generateFile(seaFileContent);				
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

	}
	
	private void generateFile(String code) {
		if (null!=jsfJavaFile){
			try{
				FileUtils.writeStringToFile(new File(jsfJavaFile), code);
			}catch (IOException e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		
		}
	}	
	
	
	private String addJsfMethod(String code){
		StringBuffer sb = new StringBuffer();
		String line = "";
		StringTokenizer st = null;
		st = new StringTokenizer(code, IResources.S_LINE_SEPARATOR);
		boolean added = false;
		while (st.hasMoreTokens()) {
			line = st.nextToken();
			if ((!added) && (line.indexOf("{")!=-1)){
				sb.append(line);
				sb.append(IResources.S_LINE_SEPARATOR);
				if (type.equals("command")){
					sb.append(IResources.JSF_COMMAND_EXECUTE);
					sb.append(IResources.S_LINE_SEPARATOR);
				}else if (type.equals("switch")){
					sb.append(IResources.JSF_SWITCH_EVALUATE);
					sb.append(IResources.S_LINE_SEPARATOR);
				}else if (type.equals("if")){
					sb.append(IResources.JSF_IF_EVALUATE);
					sb.append(IResources.S_LINE_SEPARATOR);
				}
				added = true;
			}
			else{
				sb.append(line);
				sb.append(IResources.S_LINE_SEPARATOR);				
			}
		}
		
		return sb.toString();
		
	}
	

	private String addImports(String code) {
		String importt = "";
		StringBuffer importBuffer = new StringBuffer();
		String sImports = "";
		StringBuffer resultBuffer = new StringBuffer();
		for (int i = 0; i < IResources.JSF_EXECUTE_IMPORTS.length; i++) {
			importt = IResources.JSF_EXECUTE_IMPORTS[i];
			if (code.indexOf(importt) == -1) {
				importBuffer.append("import ");
				importBuffer.append(importt);
				importBuffer.append(";");
				importBuffer.append(IResources.S_LINE_SEPARATOR);
			}
		}
		sImports = importBuffer.toString();
		StringTokenizer st = null;
		// -- añadir después del último import
		// -- las clases SEA tienen cómo mínimo
		// -- un import
		boolean last = false;
		boolean found = false;
		String line = "";
		if (!"".equals(sImports)) {
			st = new StringTokenizer(code, IResources.S_LINE_SEPARATOR);
			while (st.hasMoreTokens()) {
				line = st.nextToken();
				if (!line.trim().equals("")) {
 						if ((found)&&(!last)) {
							if (line.indexOf("import") == -1) {
								resultBuffer.append(sImports);
								resultBuffer.append(IResources.S_LINE_SEPARATOR);
								last = true;
							}
						} else if ((!found)&&(line.indexOf("import") != -1)){
							found = true;
						}
				}
				resultBuffer.append(line);
				resultBuffer.append(IResources.S_LINE_SEPARATOR);
			}
		}
		return resultBuffer.toString();
	}

}
