/*******************************************************************************
 * ViewState.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;
import java.io.FileInputStream;
import java.util.Collection;

import org.apache.commons.io.FileUtils;

import com.parser.widgets.EventView;
import com.parser.widgets.Widget;
import com.parser.widgets.WidgetPage;
import com.parser.widgets.WidgetTranslator;
import com.seatojsf.appflowmig.FlowStateNode;
import com.seatojsf.appflowmig.JSFProjectInfo;
import com.seatojsf.appflowmig.SEAProjectInfo;

public class ViewState {
	
	
	private FlowStateNode node;
	private SEAProjectInfo seaProjectInfo;
	private JSFProjectInfo jsfProjectInfo;
	private File templateDir;
	

	public ViewState(FlowStateNode node, SEAProjectInfo seaProjectInfo,
			JSFProjectInfo jsfProjectInfo, File templateDir) {
		this.node = node;
		this.seaProjectInfo = seaProjectInfo;
		this.jsfProjectInfo = jsfProjectInfo;
		this.templateDir = templateDir;
	}

	public void writeView(){
		FileInputStream fis = null;
		try{
			String viewName = node.getViewPath().replaceAll("/WEB-INF", "");
			viewName =viewName.replaceAll("\\.jsp", ".xhtml");
			File file = new File(jsfProjectInfo.getWebcontentPath() + "/view/" + viewName);
			File f = new File(seaProjectInfo.getWebcontentPath() + node.getViewPath());
			System.out.println("Processing ... " + f.getAbsolutePath());
			fis = new FileInputStream(f);
			WidgetPage wp = new WidgetPage(fis);
			wp.parsePage();
			Collection<Widget> col = wp.getWidgets();
			ViewEventParser vParser = new ViewEventParser(node);
			Collection<EventView> eventsView = vParser.parse();
			WidgetTranslator translator = new WidgetTranslator(col,templateDir,eventsView);
			String content = translator.translate();
			FileUtils.writeStringToFile(file, content);			
		}catch(Exception e){
			throw new RuntimeException(e);
		}finally{
			if (fis!=null){
				try{
					fis.close();	
				}catch (Exception e){
					throw new RuntimeException(e);		
				}
			}
		}
	}

}
