/*******************************************************************************
 * WebXmlJSF.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.xml.namespace.QName;

import org.apache.commons.io.FileUtils;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlOptions;

import com.seatojsf.appflowmig.ContextParam;
import com.seatojsf.appflowmig.EnvEntry;
import com.seatojsf.appflowmig.Filter;
import com.seatojsf.appflowmig.FilterMapping;
import com.seatojsf.appflowmig.JSFProjectInfo;
import com.seatojsf.appflowmig.Listener;
import com.seatojsf.appflowmig.Servlet;
import com.seatojsf.appflowmig.ServletMapping;
import com.seatojsf.appflowmig.WebXmlSEA;
import com.sun.java.xml.ns.javaee.DescriptionType;
import com.sun.java.xml.ns.javaee.DisplayNameType;
import com.sun.java.xml.ns.javaee.EnvEntryType;
import com.sun.java.xml.ns.javaee.EnvEntryTypeValuesType;
import com.sun.java.xml.ns.javaee.FilterMappingType;
import com.sun.java.xml.ns.javaee.FilterNameType;
import com.sun.java.xml.ns.javaee.FilterType;
import com.sun.java.xml.ns.javaee.FullyQualifiedClassType;
import com.sun.java.xml.ns.javaee.JndiNameType;
import com.sun.java.xml.ns.javaee.ListenerType;
import com.sun.java.xml.ns.javaee.ParamValueType;
import com.sun.java.xml.ns.javaee.ServletMappingType;
import com.sun.java.xml.ns.javaee.ServletNameType;
import com.sun.java.xml.ns.javaee.ServletType;
import com.sun.java.xml.ns.javaee.UrlPatternType;
import com.sun.java.xml.ns.javaee.WebAppDocument;
import com.sun.java.xml.ns.javaee.WebAppType;
import com.sun.java.xml.ns.javaee.XsdStringType;

public class WebXmlJSF {
	
	private JSFProjectInfo jsfProjectInfo;
	private WebXmlSEA dd;
	private WebAppDocument wepApp;
	WebAppType webAppType;
	
	private static Set<Listener> listenersToExclude;
	private static Set<Servlet> servletsToExclude;
	private static Set<ServletMapping> servletsMappingToExclude;
	private static Set<EnvEntry> envEntriesToExclude;
	private static Set<Filter> filtersToExclude;
	private static Set<FilterMapping> filtersMappingToExclude;
	private static Set<ContextParam> contextParamsToExclude;
	
	static{
		listenersToExclude = new HashSet<Listener>();
		servletsToExclude = new HashSet<Servlet>();
		servletsToExclude.add(new Servlet("action", null));
		servletsToExclude.add(new Servlet("sea_ajax", null));
		servletsMappingToExclude = new HashSet<ServletMapping>();
		servletsMappingToExclude.add(new ServletMapping("action", null));
		servletsMappingToExclude.add(new ServletMapping("sea_ajax", null));
		envEntriesToExclude = new HashSet<EnvEntry>();
		filtersToExclude = new HashSet<Filter>();
		filtersMappingToExclude = new HashSet<FilterMapping>();
		contextParamsToExclude = new HashSet<ContextParam>();
	}
	

	public WebXmlJSF(JSFProjectInfo jsfProjectInfo, WebXmlSEA dd) {
		this.jsfProjectInfo = jsfProjectInfo;
		this.dd = dd;
		this.wepApp = WebAppDocument.Factory.newInstance();
		this.webAppType = wepApp.addNewWebApp();		
	}

	public void writeWebXml() throws Exception{
		buildBasicWebXml();
		mergeBasicWebXmlWithSeaWebXml();
		writeFile();
	}
	
	public void addListener(Listener listener) {
		ListenerType listenerType = webAppType.addNewListener();
		FullyQualifiedClassType listenerClass = listenerType.addNewListenerClass();
		listenerClass.setStringValue(listener.getListenerClass());
	}
	
	
	public void addServletMapping(ServletMapping servletMapping) {
		ServletMappingType servletMappingType = webAppType.addNewServletMapping();
		ServletNameType servletName = servletMappingType.addNewServletName();
		servletName.setStringValue(servletMapping.getServletName());
		Collection<String> urlPatterns = servletMapping.getUrlPatterns();
		Iterator<String> it =  urlPatterns.iterator();
		String pattern = null;
		UrlPatternType url = null;
		int i = 0;
		while (it.hasNext()){
			pattern = it.next();
			url = servletMappingType.addNewUrlPattern();
			url.setStringValue(pattern);
			servletMappingType.setUrlPatternArray(i,url);
			i++;
		}
		
	}
	
	
	public void addFilterMapping(FilterMapping filterMapping) {
		FilterMappingType filterMappingType = webAppType.addNewFilterMapping();
		FilterNameType filterName = filterMappingType.addNewFilterName();
		filterName.setStringValue(filterMapping.getFilterName());
		Collection<String> urlPatterns = filterMapping.getUrlPatterns();
		Iterator<String> it =  urlPatterns.iterator();
		String pattern = null;
		UrlPatternType url = null;
		int i = 0;
		while (it.hasNext()){
			pattern = it.next();
			url = filterMappingType.addNewUrlPattern();
			url.setStringValue(pattern);
			filterMappingType.setUrlPatternArray(i,url);
			i++;
		}
		
	}	

	public void addServlet(Servlet servlet) {
		ServletType servletType = webAppType.addNewServlet();
		ServletNameType servletName = servletType.addNewServletName();
		servletName.setStringValue(servlet.getServletName());
		FullyQualifiedClassType servletClass = servletType.addNewServletClass();
		servletClass.setStringValue(servlet.getServletClass());
		servletType.setLoadOnStartup(servlet.getLoadOnStartup());
	}
	
	public void addFilter(Filter filter) {
		FilterType filterType = webAppType.addNewFilter();
		FilterNameType filterName = filterType.addNewFilterName();
		filterName.setStringValue(filter.getFilterName());
		FullyQualifiedClassType filterClass = filterType.addNewFilterClass();
		filterClass.setStringValue(filter.getFilterClass());
	}	
	

	public void addContextParam(ContextParam contextParam) {
		ParamValueType param = webAppType.addNewContextParam();
		com.sun.java.xml.ns.javaee.String name = param.addNewParamName();
		name.setStringValue(contextParam.getParamName());
		XsdStringType xsdString = param.addNewParamValue();
		xsdString.setStringValue(contextParam.getParamValue());
	}
	
	public void addEnvEntry(EnvEntry envEntry) {
		EnvEntryType entryType = webAppType.addNewEnvEntry();
		DescriptionType description = entryType.addNewDescription();
		description.setStringValue(envEntry.getDescription());
		JndiNameType name = entryType.addNewEnvEntryName();
		name.setStringValue(envEntry.getName());
		EnvEntryTypeValuesType values = entryType.addNewEnvEntryType();
		values.set(EnvEntryTypeValuesType.Enum.forString(envEntry.getType()));
		XsdStringType value = entryType.addNewEnvEntryValue();
		value.setStringValue(envEntry.getValue());
	}	



	public void addDisplayName(String name) {
		DisplayNameType displayName = webAppType.addNewDisplayName();
		displayName.setStringValue(name);
	}	

	
	
	//---------------------------------------------------------------------------------------------------------------------------------
	
	private void writeFile() throws IOException {
		XmlCursor cursor = wepApp.newCursor();
		if (cursor.toFirstChild())
		{
			cursor.setAttributeText(new QName("http://www.w3.org/2001/XMLSchema-instance","schemaLocation"), 
					"http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-app_3_0.xsd");
			cursor.toNextAttribute();
			cursor.setAttributeText(new QName("","version"), "3.0");
		}
		cursor.dispose();
		File f = new File(jsfProjectInfo.getWebcontentPath() + "/WEB-INF/web.xml");
		FileUtils.write(f, "");
		XmlOptions options = new XmlOptions();
		options.setSavePrettyPrint();
		options.setUseDefaultNamespace();
		wepApp.save(f,options);		
		//FileUtils.write(f, wepApp.toString());
	}	
	
	private void buildBasicWebXml() {
		addDisplayName(jsfProjectInfo.getProjectName());
		addContextParam(new ContextParam("javax.faces.STATE_SAVING_METHOD", "server"));
		addServlet(new Servlet("Faces Servlet", "javax.faces.webapp.FacesServlet", new Integer(1)));
		ArrayList<String> urlPatterns = new ArrayList<String>();
		urlPatterns.add("*.jsf");
		urlPatterns.add("*.xhtml");
		addServletMapping(new ServletMapping("Faces Servlet",urlPatterns));
	}	
	
	private void mergeBasicWebXmlWithSeaWebXml() throws Exception{
		addSeaListeners();
		addSeaServlets();
		addSeaServletsMapping();
		addSeaEnvEntries();
		addSeaFilters();
		addSeaFiltersMapping();
		//addSeaContextParams();

	}


	//private void addSeaContextParams() {
	//	// TODO Auto-generated method stub
	//	
	//}

	private void addSeaFiltersMapping() {
		try{
			Collection<FilterMapping> col = dd.getFiltersMapping();
			Method method = this.getClass().getMethod("addFilterMapping", FilterMapping.class);
			add(col, filtersMappingToExclude, method);			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
		
	}

	private void addSeaFilters() {
		try{
			Collection<Filter> col = dd.getFilters();
			Method method = this.getClass().getMethod("addFilter", Filter.class);
			add(col, filtersToExclude, method);			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
		
	}

	private void addSeaEnvEntries() {
		try{
			Collection<EnvEntry> col = dd.getEnvEntries();
			Method method = this.getClass().getMethod("addEnvEntry", EnvEntry.class);
			add(col, envEntriesToExclude, method);			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}

	private void addSeaServletsMapping() {
		try{
			Collection<ServletMapping> col = dd.getServletsMapping();
			Method method = this.getClass().getMethod("addServletMapping", ServletMapping.class);
			add(col, servletsMappingToExclude, method);			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}

	private void addSeaListeners() throws Exception{
		try{
			Collection<Listener> col = dd.getListeners();
			Method method = this.getClass().getMethod("addListener", Listener.class);
			add(col, listenersToExclude, method);			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}	
	
	
	private void addSeaServlets(){
		try{
			Collection<Servlet> col = dd.getServlets();
			Method method = this.getClass().getMethod("addServlet", Servlet.class);
			add(col, servletsToExclude, method);			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}
	
	private void add(@SuppressWarnings("rawtypes") Collection col, @SuppressWarnings("rawtypes") Set toExclude, Method method)throws Exception{
		@SuppressWarnings("rawtypes")
		Iterator it = col.iterator();
		Object o = null;
		while (it.hasNext()){
			o = it.next();
			if (!toExclude.contains(o)){
				method.invoke(this, o);
			}		
		}
	}







}
