/*******************************************************************************
 * XHTMLState.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.io.FileUtils;
import com.atsistemas.webbeans.model.v1.Case;
import com.atsistemas.webbeans.model.v1.IfBody;
import com.atsistemas.webbeans.model.v1.InvokeBody;
import com.atsistemas.webbeans.model.v1.State;
import com.atsistemas.webbeans.model.v1.SwitchBody;
import com.seatojsf.appflowmig.FlowStateList;
import com.seatojsf.appflowmig.FlowStateNode;


/**
 * 
 * Clase para generar un fichero .xhtml asociado a un JavaState,
 * el fichero que genera es el encargado de llamar al JavaState, 
 * el cúal no deja de ser un managedBean.
 */
public class XHTMLState {

	private FlowStateNode node;
	private String webbeanName;
	private String webcontentPath;
	private String managedBeanId;
	private FlowStateList flowStateList;


	/**
	 * Construye el objeto para poder generar el 
	 * fichero XHTML
	 * 
	 * @param node
	 * @param webbeanName
	 * @param webcontentPath
	 * @param managedBeanId
	 * @param flowStateList
	 */
	public XHTMLState(FlowStateNode node, String webcontentPath, String managedBeanId,
			FlowStateList flowStateList) {
		this.node = node;
		this.webbeanName = node.getWebbeanId();
		this.webcontentPath = webcontentPath;
		this.managedBeanId = managedBeanId;
		this.flowStateList = flowStateList;
	}

	/**
	 * Si no existe el fichero de flujo associado al node lo genera dentro de la
	 * carpeta WebContent/flow. El nombre del fichero se contruye a partir del
	 * nombre del webbean concatenando '.' más el nombre de la clase del nodo y
	 * con extensión xhtml. El ficheor esta dentro de la una carpeta según su
	 * tipo, por ejemplo si es un command estará dentro de la carpeta command.
	 * <p>
	 * Por ejemplo:
	 * .../WebContent/flow/command/webbean.com.command1.Command.xhtml
	 * 
	 */
	public void generateFile() {
		String absolutePathFile = this.getXHTMLAbsolutePath();
		// -- leer la plantilla y substituir por el redirec y el bean associado.
		String template = IResources.XHTML_TEMPLATE;
		template = template.replaceFirst("@javaManaged", getRedirec());
		File file = new File(absolutePathFile);
		if (!file.exists()) {
			generateFile(template, file);
		}
	}

	private String getXHTMLAbsolutePath() {
		XHTMLPathGenerator generator = new XHTMLPathGenerator(node);
		String relativePath = generator.getPath();
		StringBuffer sb = new StringBuffer();
		sb.append(this.webcontentPath);
		sb.append("/");
		sb.append(relativePath);
		return sb.toString();
	}

	
	
	
	private String appendRedirecValues(String childType, String key, String value, String webbeanId){
		StringBuffer sb = new StringBuffer();
		//-- key
		sb.append("\\\\'");
		sb.append(key);
		sb.append("\\\\'");
		sb.append(":");
		//-- value
		sb.append("\\\\'");
		sb.append("/");
		sb.append(childType);
		sb.append("/");
		sb.append(webbeanId);
		if ("view".equals(childType)){
			sb.append("/");	
		}
		else{
			sb.append(".");			
		}
		sb.append(value);
		sb.append(".xhtml");			
		sb.append("\\\\'");		
		return sb.toString();
	}
	


	
	
	
	//TODO OJO el tipo de los hijos no tiene que se
	//      como el padre, eso esta mal. hay que buscar
	//      al hijo y mirar de que tipo es, las vistas
	//      y los invoke se tratan de diferente manera
	private String getRedirec() {
		State st = node.getState();
		Collection<FlowStateNode> children = node.getChildren();
		StringBuffer sb = new StringBuffer();
		sb.append(managedBeanId);
		sb.append(".");
		if (st.isSetCOMMAND()) {
			Iterator<FlowStateNode> nodes = children.iterator();
			FlowStateNode child = nodes.next();
			sb.append("jsfExecute('{\\\\'type\\\\':\\\\'command\\\\',\\\\'values\\\\':{");
			if (null!=child){
				sb.append(appendRedirecValues(FlowStateNode.getType(child), "", FlowStateNode.getPackageAndClass(child),webbeanName));				
			}
		} else if (st.isSetEXECUTEVIEW()) {

		} else if (st.isSetFORK()) {
		} else if (st.isSetIF()) {
			sb.append("jsfExecute('{\\\\'type\\\\':\\\\'if\\\\',\\\\'values\\\\':{");
			IfBody body = st.getIF();
			String thenGoToState = body.getThenGoState();
			Iterator<FlowStateNode> nodes = children.iterator();
			FlowStateNode child1 = nodes.next();
			FlowStateNode child2 = nodes.next();
			String key1 = "";
			String key2 = "";
			if (child1.getState().getId().equals(thenGoToState)){
				key1 = "true";
				key2 = "false";
			}
			else{
				key1 = "false";
				key2 = "true";				
			}
			if (null!=child1){
				sb.append(appendRedirecValues(FlowStateNode.getType(child1), key1, FlowStateNode.getPackageAndClass(child1),webbeanName));
				sb.append(",");				
			}
			if (null!=child2){
				sb.append(appendRedirecValues(FlowStateNode.getType(child2), key2, FlowStateNode.getPackageAndClass(child2),webbeanName));				
			}
		} else if (st.isSetINVOKE()) {
			InvokeBody body = st.getINVOKE();
			sb.append("jsfExecute('{\\\\'type\\\\':\\\\'invoke\\\\',\\\\'values\\\\':{");
			String webbeanTarget = body.getTarget();
			FlowStateNode root = flowStateList.getFlowStateNode(webbeanTarget).getRoot();
			sb.append(appendRedirecValues(FlowStateNode.getType(root), "target", FlowStateNode.getPackageAndClass(root),webbeanTarget));
			sb.append(",");	
			FlowStateNode gotostate = FlowStateNode.getChildInAllFlow(body.getGoState() , node.getRoot(), new HashSet<Object>(),false,null);
			sb.append(appendRedirecValues(FlowStateNode.getType(gotostate), "gotostate", FlowStateNode.getPackageAndClass(gotostate),webbeanName));

			
		} else if (st.isSetRETURN()) {
			sb.append("jsfExecute('{\\\\'type\\\\':\\\\'return\\\\',\\\\'values\\\\':{");
			
		} else if (st.isSetSWITCH()) {
			sb.append("jsfExecute('{\\\\'type\\\\':\\\\'switch\\\\',\\\\'values\\\\':{");	
			SwitchBody body = st.getSWITCH();
			FlowStateNode child = null;
			Iterator<FlowStateNode> nodes = children.iterator();
			List<Case> cases = body.getCASEList();
			while (nodes.hasNext()){
				Case pointer = null;
				String key = "";
				child = nodes.next();
				boolean found = false;
				int i = 0;
				while ((!found) && (i<cases.size())){
					pointer = cases.get(i);
					if (pointer.getGoState().equals(child.getState().getId())){
						found = true;
						key = pointer.getResult();
					}
					else {
						i++;
					}
				}
				if (null!=child){
					sb.append(appendRedirecValues(FlowStateNode.getType(child), key, FlowStateNode.getPackageAndClass(child),webbeanName));
					sb.append(",");					
				}
			}
			//ADD default
			String defaultState = body.getDEFAULT().getGoState();
			child = FlowStateNode.getChild(defaultState, children);
			// Porque un state return no tiene hijos
			if (null!=child){
				sb.append(appendRedirecValues(FlowStateNode.getType(child), "default", FlowStateNode.getPackageAndClass(child),webbeanName));				
			}
		}
		sb.append("}}')");
		return sb.toString();
	}	
	
	
	

	private void generateFile(String template, File file) {
		try{
			FileUtils.writeStringToFile(file, template);			
		}catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

		

	}

}
