/*******************************************************************************
 * FlowStateNodeTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.io.File;
import java.util.Collection;

import org.junit.Assert;
import org.junit.Test;
import static org.junit.Assert.*;
import com.atsistemas.webbeans.model.v1.Flow;
import com.test.utils.FlowStateNodeUtils;


public class FlowStateNodeTest {
	
	@Test
	public void parse_newsletter_webbean()throws Exception{
		//--get states
		WebBean wp = WebBean.parse(new File("test_resources/input/files/newsletter.webbean"),
				new File("test_resources/input/files/struts-config-newsletter.xml"));
		Flow flow = wp.getFlow();
		FlowStateNode root = FlowStateNode.parse(flow,wp.getId(),wp.getViews());
		Assert.assertEquals("newsletter", root.getState().getId());
		//Hijos primer elemento newsletter
		Collection<FlowStateNode> newsletterChildren = root.getChildren();
		FlowStateNode uploadChild = FlowStateNode.getChild("Upload", newsletterChildren);
		Assert.assertEquals("Upload", uploadChild.getState().getId());
		FlowStateNode returnChild = FlowStateNode.getChild("return", newsletterChildren);
		Assert.assertEquals("return", returnChild.getState().getId());
		//Hijos elemento Upload
		Collection<FlowStateNode> uploadChildren = uploadChild.getChildren();
		FlowStateNode switchChild = FlowStateNode.getChild("Switch", uploadChildren);
		Assert.assertEquals("Switch", switchChild.getState().getId());
		//Hijos elemento Switch
		Collection<FlowStateNode> switchChildren = switchChild.getChildren();
		FlowStateNode newsletterChild = FlowStateNode.getChild("newsletter", switchChildren);
		Assert.assertEquals("newsletter", newsletterChild.getState().getId());
		returnChild = FlowStateNode.getChild("return", switchChildren);
		Assert.assertEquals("return", returnChild.getState().getId());
		FlowStateNode sendEmailChild = FlowStateNode.getChild("SendEmail", switchChildren);
		Assert.assertEquals("SendEmail", sendEmailChild.getState().getId());
		FlowStateNode addCidsImagesChild = FlowStateNode.getChild("addCidsImages", switchChildren);
		Assert.assertEquals("addCidsImages", addCidsImagesChild.getState().getId());
		//Hijos elemento SendEmail
		Collection<FlowStateNode> sendEmailChildren = sendEmailChild.getChildren();
		FlowStateNode confirmationChild = FlowStateNode.getChild("confirmation", sendEmailChildren);
		Assert.assertEquals("confirmation", confirmationChild.getState().getId());
		//Hijos elemento addCidsImages
		Collection<FlowStateNode> addCidsImagesChildren = addCidsImagesChild.getChildren();
		sendEmailChild = FlowStateNode.getChild("SendEmail", addCidsImagesChildren);
		Assert.assertEquals("SendEmail", sendEmailChild.getState().getId());
		FlowStateNode return2 = FlowStateNode.getChild("return 2", addCidsImagesChildren);
		Assert.assertEquals("return 2", return2.getState().getId());
		// Al volver al SendMail que pasa si se vuelve a mirar los hijos??
		sendEmailChildren = sendEmailChild.getChildren();
		confirmationChild = FlowStateNode.getChild("confirmation", sendEmailChildren);
		Assert.assertEquals("confirmation", confirmationChild.getState().getId());		
	}
	
	
	@Test
	public void getPackageAndClass_When_type_is_command()throws Exception{
		getClassAssert("Command", "es.plan.Command");
	}
	
	@Test
	public void getPackageAndClass_When_type_is_switch()throws Exception{
		getClassAssert("Switch", "es.plan.Switch");
	}
	
	@Test
	public void getPackageAndClass_When_type_is_if()throws Exception{
		getClassAssert("If", "es.plan.If");
	}
	
	@Test
	public void getPackageAndClass_When_type_is_return()throws Exception{
		getClassAssert("return", "es.allianz.ret.main_returnStack");
	}
	
	@Test
	public void getPackageAndClass_When_type_is_invoke()throws Exception{
		getClassAssert("Invoke", "es.allianz.invoke.mainTodetail");
	}
	
	@Test
	public void getPackageAndClass_When_type_is_view()throws Exception{
		getClassAssert("main", "main");
	}
	
	@Test
	public void getClassName_When_input_is_flowstatenode()throws Exception{
		FlowStateNode node = FlowStateNodeUtils.getNodeById_test_resources_input_PlanWeb_JavaSource_main_webbean("Invoke");
		String actual = FlowStateNode.getClassName(node);
		assertEquals("mainTodetail", actual);
	}
	
	
	
	
	
	//--------------------------------------------------------------------------------------------------------------------
	
	private void getClassAssert(String stateId, String elementAssert)throws Exception{
		FlowStateNode node = FlowStateNodeUtils.getNodeById_test_resources_input_PlanWeb_JavaSource_main_webbean(stateId);
		String actual = FlowStateNode.getPackageAndClass(node);
		assertEquals(elementAssert, actual);

	}	
	
	
	

}
