/*******************************************************************************
 * ProjectTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import static org.junit.Assert.assertEquals;


import java.io.File;
import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Test;



public class ProjectTest {
	
	@Test
	public void migrate_When_input_is_PlanWeb_sea_project()throws Exception{
		Project pr = new Project(
				new File("test_resources/input/PlanWeb"), 
				new File("test_resources/input/PlanWeb/JavaSource"), 
				new File("test_resources/input/PlanWeb/WebContent"), 
				new File("test_resources/output"),"PlanWeb",
				new File("configuration"), new File("templates/primefaces"));
		pr.migrate();
		String facesConfigFilePath = "test_resources/output/PlanWeb/WebContent/WEB-INF/faces-config.xml";
		assertFileExist(facesConfigFilePath);
		String sFacesConfig = FileUtils.readFileToString(new File(facesConfigFilePath));
		System.out.println("//------------faces-config.xml--------------");
		System.out.println(sFacesConfig);
		System.out.println("//------------------------------------------");
		validateManagedBeans(sFacesConfig);
		validateCommands(sFacesConfig);
		validateSwitches(sFacesConfig);
		validateIf(sFacesConfig);
		validateReturn(sFacesConfig);
		validateInvoke(sFacesConfig);
		validateView(sFacesConfig);
	}





	@After
	public void clean()throws Exception{
		FileUtils.deleteDirectory(new File("test_resources/output/PlanWeb"));
	}

	
	
	
	//----------------------------------------------------------------------------

	
	private void validateView(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/WebContent/view/main/customer.xhtml");
		assertFileExist("test_resources/output/PlanWeb/WebContent/view/main/list.xhtml");
		assertFileExist("test_resources/output/PlanWeb/WebContent/view/main/main.xhtml");
	}
	
	private void validateInvoke(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/allianz/invoke/mainTodetail.java");
		assertFileExist("test_resources/output/PlanWeb/WebContent/invoke/main.es.allianz.invoke.mainTodetail.xhtml");
		assertsManagedBeanInFacesConfigFile("mainTodetail","es.allianz.invoke.mainTodetail",sFacesConfig);		
	}
	
	private void validateReturn(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/allianz/ret/main_returnStack.java");
		assertFileExist("test_resources/output/PlanWeb/WebContent/return/main.es.allianz.ret.main_returnStack.xhtml");
		assertsManagedBeanInFacesConfigFile("main_returnStack","es.allianz.ret.main_returnStack",sFacesConfig);
	}	
	
	private void validateIf(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/plan/If.java");
		assertFileExist("test_resources/output/PlanWeb/WebContent/if/main.es.plan.If.xhtml");
		assertsManagedBeanInFacesConfigFile("If","es.plan.If",sFacesConfig);
	}	
	
	
	private void validateSwitches(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/plan/Switch.java");
		assertFileExist("test_resources/output/PlanWeb/WebContent/switch/main.es.plan.Switch.xhtml");
		assertsManagedBeanInFacesConfigFile("Switch","es.plan.Switch",sFacesConfig);
	}
	
	
	private void validateCommands(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/plan/Init.java");
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/plan/Command.java");
		assertFileExist("test_resources/output/PlanWeb/WebContent/command/main.es.plan.Init.xhtml");
		assertFileExist("test_resources/output/PlanWeb/WebContent/command/main.es.plan.Command.xhtml");
		assertsManagedBeanInFacesConfigFile("Init","es.plan.Init",sFacesConfig);
		assertsManagedBeanInFacesConfigFile("Command","es.plan.Command",sFacesConfig);
	}	
	
	private void validateManagedBeans(String sFacesConfig) throws Exception {
		assertFileExist("test_resources/output/PlanWeb/JavaSource/es/plan/Bean.java");
		assertsManagedBeanInFacesConfigFile("modelData","java.util.ArrayList",sFacesConfig);
		assertsManagedBeanInFacesConfigFile("nopublic","java.util.ArrayList",sFacesConfig);
		assertsManagedBeanInFacesConfigFile("bean","es.plan.Bean",sFacesConfig);
	}	
	
	public void assertFileExist(String filePath)throws Exception{
		System.out.println("Validando fichero: " + filePath);
		assertEquals(true, ((new File(filePath)).exists())); 
	}
	
	
	public void assertsManagedBeanInFacesConfigFile(String name, String clazz, String sFacesConfig){
		System.out.println("Validando bean: " + name + "::" + clazz);
		assertEquals(true, sFacesConfig.contains("<managed-bean-name>"+name+"</managed-bean-name>"));
		assertEquals(true, sFacesConfig.contains("<managed-bean-class>"+clazz+"</managed-bean-class>"));
	}
	
	
	
	
	
	

}
