/*******************************************************************************
 * WebXmlSEATest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.appflowmig;

import java.io.File;
import java.util.Collection;
import org.junit.Test;
import com.sun.java.xml.ns.j2Ee.ListenerType;
import static org.junit.Assert.assertEquals;

public class WebXmlSEATest {
	
	
	@Test
	public void parse_When_web_xml_contains_env_entry()throws Exception{
		WebXmlSEA dd = parse();
		Collection<EnvEntry> entries = dd.getEnvEntries();
		EnvEntry entry = (EnvEntry)entries.toArray()[0];
		assertEquals("Auto Generated - SDO Datasource connection to jdbc:mysql://localhost:3306/listin", entry.getDescription());
		assertEquals("MySQL listin", entry.getName());
		assertEquals("java.lang.String", entry.getType());
		assertEquals("jdbc:mysql://localhost:3306/listin,com.mysql.jdbc.Driver,listin,listin", entry.getValue());
	}
	
	@Test
	public void parse_When_web_xml_contains_filter()throws Exception{
		WebXmlSEA dd = parse();
		Collection<Filter> filters = dd.getFilters();
		Filter filter = (Filter)filters.toArray()[0];
		assertEquals("", filter.getDescription());
		assertEquals("RequestFilter", filter.getDisplayName());
		assertEquals("RequestFilter", filter.getFilterName());
		assertEquals("com.listin.filters.RequestFilter", filter.getFilterClass());
	}
	
	@Test
	public void parse_When_web_xml_contains_filter_mapping()throws Exception{
		WebXmlSEA dd = parse();
		Collection<FilterMapping> filtersMapping = dd.getFiltersMapping();
		FilterMapping filterMapping = (FilterMapping)filtersMapping.toArray()[0];
		assertEquals("RequestFilter", filterMapping.getFilterName());
		assertEquals("/*", filterMapping.getUrlPatterns().toArray()[0]);		
	}	
	
	@Test
	public void parse_When_web_xml_contains_servlet_basic_example()throws Exception{
		WebXmlSEA dd = parse();
		Collection<Servlet> servlets = dd.getServlets();
		Servlet servlet = (Servlet)servlets.toArray()[0];
		assertEquals("sea_ajax", servlet.getServletName());
		assertEquals("com.ats.webbeans.struts.AsynchronousSeaController", servlet.getServletClass());
	}
	
	@Test
	public void parse_When_web_xml_contains_servlet_init_param()throws Exception{
		WebXmlSEA dd = parse();
		Collection<Servlet> servlets = dd.getServlets();
		Servlet servlet = (Servlet)servlets.toArray()[1];
		Collection<InitParam> initParams = servlet.getInitParams();
		InitParam initParam = (InitParam)initParams.toArray()[0];
		assertEquals("config", initParam.getParamName());
		assertEquals("/WEB-INF/struts-config.xml", initParam.getParamValue());
	}
	
	@Test
	public void parse_When_web_xml_contains_servlet_load_on_startup()throws Exception{
		WebXmlSEA dd = parse();
		Collection<Servlet> servlets = dd.getServlets();
		Servlet servlet = (Servlet)servlets.toArray()[1];
		int loadOnStartup = servlet.getLoadOnStartup();
		assertEquals(2, loadOnStartup);
	}
	
	@Test
	public void parse_When_web_xml_contains_servlet_mapping()throws Exception{
		WebXmlSEA dd = parse();
		Collection<ServletMapping> servletsMapping = dd.getServletsMapping();
		ServletMapping servletMapping = (ServletMapping)servletsMapping.toArray()[0];
		assertEquals("sea_ajax", servletMapping.getServletName());
		assertEquals("/ajax.sea", servletMapping.getUrlPatterns().toArray()[0]);
	}
	
	@Test
	public void parse_When_web_xml_contains_listener_PersistenceAppListener()throws Exception{
		WebXmlSEA dd = parse();
		Collection<Listener> listeners = dd.getListeners();
		Listener listener = (Listener)listeners.toArray()[0];
		assertEquals("com.listin.jpa.PersistenceAppListener", listener.getListenerClass());
	}	
	

	
	
	
	//-------------------------------------------------------------------
	
	private WebXmlSEA parse()throws Exception{
		File webXml = new File("test_resources/input/files/web.xml");
		WebXmlSEA dd = WebXmlSEA.parse(webXml);
		return dd;
	}
	
	

}
