/*******************************************************************************
 * CopyConfigurationFilesTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;

import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Test;

import com.seatojsf.appflowmig.JSFProjectInfo;
import static org.junit.Assert.*;


public class CopyConfigurationFilesTest {
	
	@Test
	public void copyProject_When_is_simple_test()throws Exception{
		CopyConfigurationFiles copy = createObject();
		copy.copyProject();
		assertExists("test_resources/output/PlanWeb/.project");
		assertFileContains("test_resources/output/PlanWeb/.project", "PlanWeb");
	}
	
	@Test
	public void copyClasspath_When_is_simple_test()throws Exception{
		CopyConfigurationFiles copy = createObject();
		copy.copyClasspath();
		assertExists("test_resources/output/PlanWeb/.classpath");
	}
	
	
	@Test
	public void copySettings_When_is_simple_test()throws Exception{
		CopyConfigurationFiles copy = createObject();
		copy.copySettings();
		assertExists("test_resources/output/PlanWeb/.settings/.jsdtscope");
		assertExists("test_resources/output/PlanWeb/.settings/org.eclipse.jdt.apt.core.prefs");
		assertExists("test_resources/output/PlanWeb/.settings/org.eclipse.jdt.core.prefs");
		assertExists("test_resources/output/PlanWeb/.settings/org.eclipse.wst.common.component");
		assertFileContains("test_resources/output/PlanWeb/.settings/org.eclipse.wst.common.component", "PlanWeb");
		assertExists("test_resources/output/PlanWeb/.settings/org.eclipse.wst.common.project.facet.core.xml");
		assertExists("test_resources/output/PlanWeb/.settings/org.eclipse.wst.jsdt.ui.superType.container");
		assertExists("test_resources/output/PlanWeb/.settings/org.eclipse.wst.jsdt.ui.superType.name");
	}
	
	
	@Test
	public void copyLib_When_is_simple_test()throws Exception{
		CopyConfigurationFiles copy = createObject();
		copy.copyLib();
		assertExists("test_resources/output/PlanWeb/WebContent/WEB-INF/lib/seatojsfapi.jar");
	}	
	
	@After
	public void clean()throws Exception{
		FileUtils.deleteDirectory(new File("test_resources/output/PlanWeb"));
	}	
	
	//---------------------------------------------------------------------------------------------------------------------

	private CopyConfigurationFiles createObject() {
		JSFProjectInfo jsfProjectInfo = new JSFProjectInfo("PlanWeb",new File("test_resources/output"));
		CopyConfigurationFiles copy = new CopyConfigurationFiles(
				new File("configuration"),
				jsfProjectInfo);
		return copy;
	}
	
	private void assertExists(String path)throws Exception{
		File file = new File(path);
		assertEquals(true, file.exists());		
	}
	
	private void assertFileContains(String path, String text)throws Exception{
		File file = new File(path);
		String content = FileUtils.readFileToString(file);
		boolean contains = content.contains(text);
		assertEquals(true, contains);		
	}	
	

	
	
	

}
