/*******************************************************************************
 * JavaStateTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import static org.junit.Assert.*;

import java.io.File;
import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Test;
import com.seatojsf.appflowmig.FlowStateNode;
import com.seatojsf.appflowmig.JSFProjectInfo;
import com.seatojsf.appflowmig.SEAProjectInfo;
import com.test.utils.FlowStateNodeUtils;


public class JavaStateTest {

	
	
	@Test
	public void generateJSFJavaFile_When_input_is_sea_command()throws Exception{
		String fileContents = getFileContents("Command","test_resources/output/PlanWeb/JavaSource/es/plan/Command.java");
		assertEquals(true, fileContents.contains("jsfExecute"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext"));
		assertEquals(true, fileContents.contains("javax.servlet.http.HttpServletRequest"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext;"));		
	}
	
	
	@Test
	public void generateJSFJavaFile_When_input_is_sea_if()throws Exception{
		String fileContents = getFileContents("If","test_resources/output/PlanWeb/JavaSource/es/plan/If.java");
		assertEquals(true, fileContents.contains("jsfExecute"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext"));
		assertEquals(true, fileContents.contains("javax.servlet.http.HttpServletRequest"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext;"));		
	}
	
	@Test
	public void generateJSFJavaFile_When_input_is_sea_return()throws Exception{
		String fileContents = getFileContents("return","test_resources/output/PlanWeb/JavaSource/es/allianz/ret/main_returnStack.java");
		assertEquals(true, fileContents.contains("jsfExecute"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext"));
		assertEquals(true, fileContents.contains("javax.servlet.http.HttpServletRequest"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext;"));		
	}
	
	@Test
	public void generateJSFJavaFile_When_input_is_sea_invoke()throws Exception{
		String fileContents = getFileContents("Invoke","test_resources/output/PlanWeb/JavaSource/es/allianz/invoke/mainTodetail.java");
		assertEquals(true, fileContents.contains("jsfExecute"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext"));
		assertEquals(true, fileContents.contains("javax.servlet.http.HttpServletRequest"));
		assertEquals(true, fileContents.contains("javax.faces.context.FacesContext;"));		
	}	
	
	
	
	
	
	
	@After
	public void clean()throws Exception{
		FileUtils.deleteDirectory(new File("test_resources/output/PlanWeb"));
	}
	
	//---------------------------------------------------------------------------------------------------------------------------
	
	
	private String getFileContents(String nodeId, String jsfFilePath)throws Exception{
		FlowStateNode node = FlowStateNodeUtils.getNodeById_test_resources_input_PlanWeb_JavaSource_main_webbean(nodeId);
		SEAProjectInfo seaProjectInfo = new SEAProjectInfo(new File("test_resources/input/PlanWeb"), 
				new File("test_resources/input/PlanWeb/JavaSource"), 
				new File("test_resources/input/PlanWeb/WebContent"));
		JSFProjectInfo jsfProjectInfo = new JSFProjectInfo("PlanWeb",new File("test_resources/output"));	
		JavaState jstate = new JavaState(node, seaProjectInfo, jsfProjectInfo);
		jstate.generateJSFJavaFile();
		return FileUtils.readFileToString(new File(jsfFilePath));
	}
	

}
