/*******************************************************************************
 * ViewEventParserTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;
import static org.junit.Assert.*;

import java.util.Collection;
import java.util.HashSet;
import org.junit.Test;
import com.atsistemas.webbeans.model.v1.Flow;
import com.parser.widgets.EventView;
import com.seatojsf.appflowmig.FlowStateNode;
import com.seatojsf.appflowmig.WebBean;
import com.test.utils.WebBeanParserUtilsTest;

public class ViewEventParserTest {
	
	@Test
	public void parse_When_view_has_one_command_outgoing()throws Exception{
		WebBean wp = WebBeanParserUtilsTest.parse("test_resources/input/files/alta.webbean", 
				"test_resources/input/files/struts-config-alta.xml");
		Flow flow = wp.getFlow();
		//-- El primer elemento de flujo es una vista.
		FlowStateNode view = FlowStateNode.parse(flow, "alta", wp.getViews());
		ViewEventParser parser = new ViewEventParser(view);
		Collection<EventView> eventViewCol = parser.parse();
		EventView detalle = (EventView)eventViewCol.toArray()[0];
		assertEquals("insert", detalle.getEvent());
		assertEquals("/command/alta.com.listin.commands.InsertarCommand.xhtml",detalle.getViewXHTML());
	}
	
	
	@Test
	public void parse_When_view_has_two_outgoing_switch_return()throws Exception{
		WebBean wp = WebBeanParserUtilsTest.parse("test_resources/input/PlanWeb/JavaSource/main.webbean", 
				"test_resources/input/PlanWeb/WebContent/WEB-INF/struts-config-main.xml");
		Flow flow = wp.getFlow();		
		FlowStateNode root = FlowStateNode.parse(flow, "main", wp.getViews());
		FlowStateNode view = FlowStateNode.getChildInAllFlow("main", root, new HashSet<Object>(), false, null);
		ViewEventParser parser = new ViewEventParser(view);
		Collection<EventView> eventViewCol = parser.parse();
		assertEquals(2, eventViewCol.size());
		EventView zwitch = (EventView)eventViewCol.toArray()[0];
		EventView exit = (EventView)eventViewCol.toArray()[1];
		assertEquals("switch", zwitch.getEvent());
		assertEquals("/switch/main.es.plan.Switch.xhtml",zwitch.getViewXHTML());
		assertEquals("exit", exit.getEvent());
		assertEquals("/return/main.es.allianz.ret.main_returnStack.xhtml",exit.getViewXHTML());
	}	
	
	

}
