/*******************************************************************************
 * WebXmlJSFTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import java.io.File;
import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Test;
import com.seatojsf.appflowmig.JSFProjectInfo;
import com.seatojsf.appflowmig.WebXmlSEA;
import com.seatojsf.parsers.WebXmlJSF;

import static org.junit.Assert.assertEquals;

public class WebXmlJSFTest {
	
	@Test
	public void writeWebXml_When_display_name_is_project_name()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertContains("<display-name>PlanWeb</display-name>",sWebXml);
	}

	@Test
	public void writeWebXml_When_add_context_param_tags()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertContains("<context-param>",sWebXml);
		assertContains("<param-name>javax.faces.STATE_SAVING_METHOD</param-name>",sWebXml);
		assertContains("<param-value>server</param-value>",sWebXml);
		assertContains("<param-value>server</param-value>",sWebXml);
		assertContains("</context-param>",sWebXml);
	}
	
	@Test
	public void writeWebXml_When_add_servlet_tags()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertContains("<servlet>",sWebXml);
		assertContains("<servlet-name>Faces Servlet</servlet-name>",sWebXml);
		assertContains("<servlet-class>javax.faces.webapp.FacesServlet</servlet-class>",sWebXml);
		assertContains("<load-on-startup>1</load-on-startup>",sWebXml);
		assertContains("</servlet>",sWebXml);
	}
	
	@Test
	public void writeWebXml_When_add_servlet_mapping_tags()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertContains("<servlet-mapping>",sWebXml);
		assertContains("<servlet-name>Faces Servlet</servlet-name>",sWebXml);
		assertContains("<url-pattern>*.jsf</url-pattern>",sWebXml);
		assertContains("<url-pattern>*.xhtml</url-pattern>",sWebXml);
		assertContains("</servlet-mapping>",sWebXml);		
	}
	
	
	@Test
	public void writeWebXml_When_merge_with_sea_web_xml_contains_listener_PersistenceAppListener()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertContains("<listener>",sWebXml);
		assertContains("<listener-class>com.listin.jpa.PersistenceAppListener</listener-class>",sWebXml);
		assertContains("</listener>",sWebXml);
	}
	
	@Test
	public void writeWebXml_When_merge_with_sea_web_xml_contains_env_entry_JDBC()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertContains("<env-entry>",sWebXml);
		assertContains("<description>Auto Generated - SDO Datasource connection to jdbc:mysql://localhost:3306/listin</description>",sWebXml);
		assertContains("<env-entry-type>java.lang.String</env-entry-type>",sWebXml);
		assertContains("<env-entry-value>jdbc:mysql://localhost:3306/listin,com.mysql.jdbc.Driver,listin,listin</env-entry-value>",sWebXml);
		assertContains("</env-entry>",sWebXml);
	}	
	
	@Test
	public void writeWebXml_When_merge_with_sea_web_xml_not_contains_servlet_action()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertNotContains("<display-name>Action Servlet</display-name>",sWebXml);
		assertNotContains("<servlet-name>action</servlet-name>",sWebXml);
		assertNotContains("<servlet-class>org.apache.struts.action.ActionServlet</servlet-class>",sWebXml);
	}
	
	@Test
	public void writeWebXml_When_merge_with_sea_web_xml_not_contains_servlet_mapping_action()throws Exception{
		String sWebXml = writeWebXml("test_resources/input/files/web.xml");
		assertNotContains("<servlet-name>action</servlet-name>",sWebXml);
		assertNotContains("<url-pattern>*.do</url-pattern>",sWebXml);
	}	
	
	@After
	public void clean()throws Exception{
		FileUtils.deleteDirectory(new File("test_resources/output/PlanWeb"));
	}	
	
	
	//-----------------------------------------------------------------------------------------

	
	private String writeWebXml(String webXmlPath) throws Exception {
		File webXml = new File(webXmlPath);
		WebXmlSEA dd = WebXmlSEA.parse(webXml);
		JSFProjectInfo jsfProjectInfo = new JSFProjectInfo("PlanWeb",new File("test_resources/output"));
		WebXmlJSF webXmlJsf = new WebXmlJSF(jsfProjectInfo, dd);
		webXmlJsf.writeWebXml();
		return FileUtils.readFileToString(
				new File("test_resources/output/PlanWeb/WebContent/WEB-INF/web.xml"));
	}
	
	
	private void assertContains(String text, String content){
		System.out.println("Validando si contiene el texto:: " + text);
		boolean contains = content.contains(text);
		assertEquals(true, contains);		
	}
	
	private void assertNotContains(String text, String content){
		System.out.println("Validando si no contiene el texto:: " + text);
		boolean contains = content.contains(text);
		assertEquals(false, contains);		
	}	
	

}
