/*******************************************************************************
 * XHTMLPathGeneratorTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.seatojsf.parsers;

import static org.junit.Assert.*;

import org.junit.Test;
import com.seatojsf.appflowmig.FlowStateNode;
import com.test.utils.FlowStateNodeUtils;



public class XHTMLPathGeneratorTest {
	
	@Test
	public void getPath_When_element_type_is_command()throws Exception{
		assertGetPath("Command", "/command/main.es.plan.Command.xhtml");
	}
	
	@Test
	public void getPath_When_element_type_is_switch()throws Exception{
		assertGetPath("Switch", "/switch/main.es.plan.Switch.xhtml");
	}
	
	@Test
	public void getPath_When_element_type_is_if()throws Exception{
		assertGetPath("If", "/if/main.es.plan.If.xhtml");
	}
	
	@Test
	public void getPath_When_element_type_is_invoke()throws Exception{
		assertGetPath("Invoke", "/invoke/main.es.allianz.invoke.mainTodetail.xhtml");
	}
	
	@Test
	public void getPath_When_element_type_is_return()throws Exception{
		assertGetPath("return", "/return/main.es.allianz.ret.main_returnStack.xhtml");
	}
	
	@Test
	public void getPath_When_element_type_is_view()throws Exception{
		assertGetPath("customer", "/view/main.customer.xhtml");
	}
	
	
	//--------------------------------------------------------------------------------------------------------------------
	
	
	private void assertGetPath(String nodeId, String expected)throws Exception{
		FlowStateNode node = FlowStateNodeUtils.getNodeById_test_resources_input_PlanWeb_JavaSource_main_webbean(nodeId);
		XHTMLPathGenerator generator = new XHTMLPathGenerator(node);
		assertEquals(expected, generator.getPath());		
	}
	

	
	

}
