/*******************************************************************************
 * WidgetParser.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.parser.widgets;


import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;


/**
 * 
 * Analiza una página creada con
 * widgets
 *
 */
public class WidgetParser {
	

	private static final String lineSeparator = System.getProperty("line.separator");
	private String widgetText;
	
	private int countOpen = 0;
	private int countClose = 0;
	
	
	
	public WidgetParser(String widgetText){
		this.widgetText = widgetText;
	}
	
	private boolean isWidget(String token){
		return (token.indexOf("<widget:")!=-1);
	}
	
	private boolean isJspDirective(String token){
		return (token.indexOf("<%@")!=-1);
	}	
	
	private boolean isWidgetCode(String token){
		return (token.indexOf("<widget:code")!=-1);
	}
	
	private boolean isHtmlComment(String token){
		return (token.indexOf("<!--")!=-1);
	}
	
	private boolean isStartWidget(String token){
		return (!token.equals("/>")); 
	}
	
	private boolean isJspScript(String token){
		return (token.indexOf("<%")!=-1) && (token.indexOf("<%=")==-1); 
	}	
	
	
	
	public Collection<Widget> parse(){
		StringTokenizer st = new StringTokenizer(widgetText," "+lineSeparator+"\t");
		String token = "";
		Widget pointer = null;
		Collection<Widget> col = new ArrayList<Widget>();
		while (st.hasMoreTokens()){
			token = st.nextToken();
			//NOTA: Se entiende que todo lo que no es final "/>"
			//      es principio, por ejemplo las directivas de JSP y 
			//      los script de JSP también son principio de widget.
			if (isStartWidget(token)){
				if ( (isWidget(token)) || (isJspDirective(token)) || (isJspScript(token)) || (isHtmlComment(token)) ){
					if (isJspDirective(token)){
						pointer = createWidget(token, pointer, col, "jsp_directive");
						pointer = processTag(st, token, pointer, "<%@", "%>");
						
					}
					else if (isJspScript(token)){
						pointer = createWidget(token, pointer, col, "jsp_script");
						pointer = processTag(st, token, pointer, "<%", "%>");
					}
					else if (isHtmlComment(token)){
						pointer = createWidget(token, pointer, col, "html_comment");
						pointer = processTag(st, token, pointer, "<!--", "-->");						
					}
					else{
						pointer = createWidget(token, pointer, col, getWidgetType(token));
						if (isWidgetCode(token)){
							pointer = processTag(st, token, pointer, "<widget:code>", "</widget:code>");
						}
					}
				}
				else{
					pointer = endWidget(st, token, pointer);					
				}
			}
			else{
				pointer = pointer.getParent();
				countClose = countClose +1;				
			}
			
		}
		return col;
	}

	private Widget endWidget(StringTokenizer st, String token, Widget pointer) {
		boolean isEndOfWidget = false;
		boolean isWidgetClose = false;
		while ( (!token.endsWith("\"")) && (!isEndOfWidget) ){
			if ( (token.indexOf(">")!=-1) ){
				isEndOfWidget = true;
				if (token.trim().startsWith("</")){
					token = "";
					isWidgetClose = true;
				}
				else{
					
					if (token.indexOf("/")!=-1){
						isWidgetClose = true;
					}
					if (token.indexOf("%>")!=-1){
						int index = token.indexOf("%>");
						token = token.substring(0,index+3);
						token = token.trim();						
					}else{
						int index = token.indexOf(">");
						token = token.substring(0,index);
						token = token.trim();						
					}

				}
			}
			else{
				token = token.concat(" ").concat(st.nextToken());							
			}
		}
		StringTokenizer stAt = new StringTokenizer(token,"=");
		if (!"".equals(token)){
			String name = stAt.nextToken();
			name = name.trim();
			String value = stAt.nextToken();
			if (value.contains("<%")){
				value = value + "=" + stAt.nextToken();
			}
			value = removeLeftAndRightInvalidChars(value);
			pointer.addAttribute(new Attribute(name, value));
		}
		if (isWidgetClose){
			pointer = pointer.getParent();
			countClose = countClose +1;
		}
		return pointer;
	}



	
	private Widget processTag(StringTokenizer st, String token,
			Widget pointer, String startTag, String endTag){
		String widgetCode = token;
		while (!widgetCode.endsWith(endTag)){
			widgetCode = widgetCode.concat(" ").concat(st.nextToken());
		}
		int start = widgetCode.indexOf(startTag);
		start = start + startTag.length();
		int end = widgetCode.indexOf(endTag);
		String text = widgetCode.substring(start, end);
		pointer.setText(text);
		pointer = pointer.getParent();
		countClose = countClose +1;
		return pointer;		
	}
	
	
	private String getWidgetType(String token){
		String type = "";
		type = token.replaceAll("<widget:", "");
		if (type.endsWith(">")){
			type = type.replaceFirst(">", "");
		}
		return type;
	}



	private Widget createWidget(String token, Widget pointer, Collection<Widget> col, String type) {
		Widget widget = new Widget();
		widget.setType(type);
		if (countOpen==countClose){
			col.add(widget);
			countOpen = countOpen + 1;
			pointer = widget;
		}
		else{
			widget.setParent(pointer);
			pointer.addChild(widget);
			pointer = widget;
			countOpen = countOpen + 1;
		}
		return pointer;
	}
	
	
	private String removeLeftInvalidChars(String word){
		while (!word.startsWith("\"")){
			word = word.substring(1,word.length());
		}
		if (word.startsWith("\"")){
			word = word.substring(1,word.length());
		}
		return word;
	}

	private String removeRightInvalidChars(String word){
		int size = -1;
		while (!word.endsWith("\"")){
			size = word.length();
			word = word.substring(0,size-1);
		}
		if (word.endsWith("\"")){
			size = word.length();
			word = word.substring(0,size-1);		
		}
		return word;
	}

	private String removeLeftAndRightInvalidChars(String word){
		word = removeLeftInvalidChars(word);
		word = removeRightInvalidChars(word);
		return word;
	}	
	
	
	
	
	

}
