/*******************************************************************************
 * WidgetTranslator.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.parser.widgets;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.commons.io.FileUtils;

import com.parser.widgets.EquivalentAttr.LevelAttributeName;

/**
 * 
 * Traduce una página de widgets
 * mediante las plantillas que se le
 * pasen.
 *
 */
public class WidgetTranslator {
	
	
	private Collection<Widget> widgets;
	private File templateDir;
	private Collection<EventView> eventsView;
	
	private StringBuffer sb;

	public WidgetTranslator(Collection<Widget> widgets, File templateDir, 
			Collection<EventView> eventsView) {
		this.widgets = widgets;
		this.templateDir = templateDir;
		this.eventsView = eventsView;
		this.sb = new StringBuffer();
	}

	public String translate()throws IOException{
		File header = new File(templateDir + "/header.template");
		if (header.exists()){
			String sHeader = FileUtils.readFileToString(header);
			sb.append(sHeader);
		}
		
		processChildren(widgets);
		return sb.toString();
	}
	
	
	private void processWidget(Widget widget)throws IOException{
		String type = widget.getType();
		Collection<Attribute> wAttributes = widget.getAttributes();
		File template = new File(templateDir+"/"+type+"_start.template");
		if (template.exists()){
			String sTemplate = FileUtils.readFileToString(template);
			File properties = new File(templateDir+"/"+type+"_attributes.properties");
			if (properties.exists()){
				Set<String> attrLangs = new HashSet<String>();
				//      mirar si existe el properties attr_lang para el widget
				//      si existe cargarlo en una hash para tenerlo en cuenta
				//      a la hora de obtener el valor
				File attrLang = new File(templateDir+"/"+type+"_attr_lang.properties");
				if (attrLang.exists()){
					attrLangs = loadAttrLangs(attrLang);
				}
				Properties prop = IOUtils.readPropertiesFile(templateDir+"/"+type+"_attributes.properties");
				Object[] keys =  prop.keySet().toArray();
				EquivalentAttr eqAttr = null;
				String key = null;
				HashMap<String, String> attrs = new HashMap<String, String>();
				LevelAttributeName leAttr = null;
				String name = null;
				String level = null;
				String pointer = null;
				for (int i=0;i<keys.length;i++){
					key = (String)keys[i];
					String wAttrValue = "";
					if (key.contains("+")){
						//-- Entonces el valor es la suma de los dos atributos
						//   se podría arreglar definiendo un buen lenguaje
						//   para los propeties...
						StringTokenizer st = new StringTokenizer(key,"+");
						String value1 = getAttributeValue(wAttributes,st.nextToken(),attrLangs);
						String value2 = getAttributeValue(wAttributes,st.nextToken(),attrLangs);
						wAttrValue = "#{" + value1 + "." + value2 + "}";
					}
					else{
						wAttrValue = getAttributeValue(wAttributes,key,attrLangs);
					}
					//-- Chapuza para el caso del tableColumn
					//   se podría corregir habilitando un lenguaje para
					//   los properties.
					if ( ("tableColumn".equals(type)) && ("value".equals(key)) ){
						wAttrValue = "#{item." + wAttrValue + "}";
					}
					if (null!=wAttrValue){
						eqAttr = new EquivalentAttr(key,prop.getProperty(key));
						Collection<LevelAttributeName> colAtrr = eqAttr.getLevelAtributesNames();
						Iterator<LevelAttributeName> it = colAtrr.iterator();
						while (it.hasNext()){
							leAttr = it.next();
							level = leAttr.getLevel();
							name = leAttr.getAttributeName();
							if (attrs.containsKey(level)){
								pointer = attrs.get(level);
								pointer = pointer + " ";
							}
							else{
								pointer = "";
							}
							pointer = pointer + name + "=" + "\""+wAttrValue+"\"";
							attrs.put(level, pointer);
						}						
					}
				}
				Object[] levelKeys = attrs.keySet().toArray();
				String levelKey = "";
				for (int i=0;i<levelKeys.length;i++){
					levelKey = (String)levelKeys[i];
					sTemplate = sTemplate.replaceAll("#"+levelKey, attrs.get(levelKey));
				}
			}
			sb.append(sTemplate);
		}
	}
	
	


	private Set<String> loadAttrLangs(File attrLang) throws IOException{
		Set<String> attrLangs = new HashSet<String>();
		Properties prop = IOUtils.readPropertiesFile(attrLang.getAbsolutePath());
		String attrs = prop.getProperty("attrs");
		if (null!=attrs){
			attrs = attrs.trim();
			if (!attrs.equals("")){
				if (attrs.contains(",")){
					StringTokenizer st = new StringTokenizer(attrs,",");
					while(st.hasMoreElements()){
						attrLangs.add(st.nextToken());
					}
				}
				else{
					attrLangs.add(attrs);
				}
			}
		}


		return attrLangs;
	}

	private void processChildren(Collection<Widget> tags)throws IOException{
		Iterator<Widget> it = tags.iterator();
		Widget widget = null;
		Collection<Widget> children = null;
		while (it.hasNext()){
			widget = it.next();
			processWidget(widget);
			children = widget.getChildren();
			processChildren(children);
			//-- end widget
			File file = new File(templateDir+"/"+widget.getType()+"_end.template");
			if (file.exists()){
				String sEnd = FileUtils.readFileToString(file);
				sb.append(sEnd);
			}
			
			
		}
	}
	
	
	private String getAttributeValue(Collection<Attribute> attributes, String name, Set<String> attrLangs){
		String s = null;
		Iterator<Attribute> it = attributes.iterator();
		boolean found = false;
		Attribute attr = null;
		while ( (it.hasNext()) && (!found) ){
			attr = it.next();
			if (name.equals(attr.getName())){
				found = true;
				s = attr.getValue();
			}
		}
		if (found){
			if (attrLangs.contains(attr.getName())){
				s = "#{msg['" + s + "']}";
			}
			if (s.contains("sendSEAEvent")){
				s = getXHTMLinEventsView(getEvent(s));
			}
			//-- Chapuza para los modeldata por
			//    ejemplo el table, se podría corregir
			//    habilitando un lenguaje para los properties...
			if ("modelData".equals(name)){
				s = s.replaceFirst("<%=", "");
				s = s.replaceFirst("%>", "");
				s = s.trim();
				s = "#{" + s + "}";
			}
			
		}
		return s;
	}
	
	
	private String getXHTMLinEventsView(String event){
		Iterator<EventView> it = eventsView.iterator();
		boolean found = false;
		EventView eventView = null;
		String s = "";
		while ((it.hasNext()) && (!found)){
			eventView = it.next();
			if (event.equals(eventView.getEvent())){
				s = eventView.getViewXHTML();
				found = true;
			}
		}
		return s;
	}
	
	
	public static String getEvent(String seaEvent){
		String s = "";
		StringTokenizer st = new StringTokenizer(seaEvent,"'");
		st.nextToken();
		s = st.nextToken();
		return s;
	}
	

}
