/*******************************************************************************
 * WidgetParserTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.parser.widgets;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import org.apache.commons.io.FileUtils;
import static org.junit.Assert.*;
import org.junit.Test;

import com.parser.widgets.Attribute;
import com.parser.widgets.Widget;
import com.parser.widgets.WidgetParser;


public class WidgetParserTest {
	
	@Test
	public void parser_When_parse_input_text_return_type_Text() throws Exception{
		Collection<Widget> col = parse("test_resources/input/inputtext.txt");
		assertEquals(1, col.size());
		Widget wText = (Widget)col.toArray()[0];
		assertEquals("inputText", wText.getType());
	}
	
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes() throws Exception{
		parser_input_text("test_resources/input/inputtext.txt");
	}
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes_more_lines() throws Exception{
		parser_input_text("test_resources/input/inputtext_more_lines.txt");
	}
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes_more_lines_and_spaces() throws Exception{
		parser_input_text("test_resources/input/inputtext_more_lines_and_spaces.txt");
	}
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes_more_lines_and_spaces_in_attribute() throws Exception{
		Collection<Widget> col = parse("test_resources/input/inputtext_more_lines_and_spaces_in_attribute.txt");
		Widget wText = (Widget)col.toArray()[0];
		assertEquals("inputText", wText.getType());
		Collection<Attribute> attributes = wText.getAttributes();
		assertAttribute(attributes,"id","buscar texto ");
		assertAttribute(attributes,"label","label.buscar");
		assertAttribute(attributes,"beanName","buscarBean");
		assertAttribute(attributes,"beanProperty","texto");
	}
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes_and_test_include_end_tag() throws Exception{
		parser_input_text("test_resources/input/inputtext_end_tag.txt");
	}	
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes_and_test_include_end_tag_and_spaces() throws Exception{
		parser_input_text("test_resources/input/inputtext_end_tag_and_spaces.txt");
	}
	
	@Test
	public void parser_When_parse_input_text_return_type_Text_and_attributes_and_test_include_end_tag_and_spaces_different_line() throws Exception{
		parser_input_text("test_resources/input/inputtext_end_tag_and_spaces_different_line.txt");
	}	
	
	
	@Test
	public void parser_When_parse_sectionButton_return_type_sectionButton_and_parse_attributes() throws Exception{
		Collection<Widget> col = parse("test_resources/input/sectionButton.txt");
		assertEquals(1, col.size());
		Widget wText = (Widget)col.toArray()[0];
		Collection<Attribute> attributes = wText.getAttributes();
		assertAttribute(attributes,"value","label.buscar");
		assertAttribute(attributes,"target","sendSEAEvent(mainForm,'buscar')");
	}
	
	@Test
	public void parser_When_parse_footerButtonBar_with_children() throws Exception{
		Collection<Widget> col = parse("test_resources/input/footerButtonBar.txt");
		assertEquals(1, col.size());
		Widget widget = (Widget)col.toArray()[0];
		assertEquals("footerButtonBar", widget.getType());
		Collection<Widget> children =  widget.getChildren();
		assertEquals(1, children.size());
		Widget child = (Widget)children.toArray()[0];
		assertEquals("footerButton", child.getType());
		Collection<Attribute> attributes = child.getAttributes();
		assertAttribute(attributes,"value","label.close");
		assertAttribute(attributes,"target","window.close()");
	}	
	
	
	@Test
	public void parser_When_parse_section() throws Exception{
		Collection<Widget> col = parse("test_resources/input/section.txt");
		assertEquals(1, col.size());
		Widget widget = (Widget)col.toArray()[0];
		Collection<Widget> children =  widget.getChildren();
		assertEquals(2, children.size());
		Collection<Attribute> attributes = widget.getAttributes();
		assertAttribute(attributes,"id","sectionPaginaCompleta");
		assertAttribute(attributes,"title","label.listin");
		Widget child = (Widget)children.toArray()[0];
		attributes = child.getAttributes();
		assertAttribute(attributes,"id","buscar");
		assertAttribute(attributes,"label","label.buscar");
		assertAttribute(attributes,"beanName","buscarBean");
		assertAttribute(attributes,"beanProperty","texto");
		Widget child1 = (Widget)children.toArray()[1];
		attributes = child1.getAttributes();
		assertEquals(0, attributes.size());
		Widget child12 = (Widget)child1.getChildren().toArray()[0];
		attributes = child12.getAttributes();
		assertAttribute(attributes,"value","label.buscar");
		assertAttribute(attributes,"target","sendSEAEvent(mainForm,'buscar')");
	}
	
	
	
	@Test
	public void parser_When_parse_two_widgets_errors_and_page() throws Exception{
		Collection<Widget> col = parse("test_resources/input/errors_apppage.txt");
		assertEquals(2, col.size());
		Widget widget1 = (Widget)col.toArray()[0];
		Widget widget2 = (Widget)col.toArray()[1];
		assertEquals("errors", widget1.getType());
		assertEquals("appPage", widget2.getType());
		Collection<Widget> children1 = widget1.getChildren();
		assertEquals(0, children1.size());
		Collection<Widget> children2 = widget2.getChildren();
		assertEquals(0, children2.size());
		Collection<Attribute> attributes1 = widget1.getAttributes();
		Collection<Attribute> attributes2 = widget2.getAttributes();
		assertEquals(1, attributes1.size());
		assertEquals(2, attributes2.size());
		assertAttribute(attributes1,"formId","mainForm");
		assertAttribute(attributes2,"appTitle","");
		assertAttribute(attributes2,"type","inpage");
	}
	
	
	@Test
	public void parser_When_parse_code() throws Exception{
		Collection<Widget> col = parse("test_resources/input/code.txt");
		Widget widget = (Widget)col.toArray()[0];
		assertEquals(1, col.size());
		assertEquals("code", widget.getType());
		assertEquals(" <input type=\"hidden\" value = \"\"/> ", widget.getText());
	}
	
	
	@Test
	public void parser_When_parse_form_with_code() throws Exception{
		Collection<Widget> col = parse("test_resources/input/form.txt");
		Widget widget = (Widget)col.toArray()[0];
		assertEquals(1, col.size());
		assertEquals("form", widget.getType());
		Collection<Widget> children = widget.getChildren();
		assertEquals(3, children.size());
		Widget widget1 = (Widget)children.toArray()[0];
		Widget widget2 = (Widget)children.toArray()[1];
		Widget widget3 = (Widget)children.toArray()[2];
		assertEquals("section", widget1.getType());
		assertEquals("code", widget2.getType());
		assertEquals("inputText", widget3.getType());
	}
	
	
	@Test
	public void parser_When_parse_jsp_directive() throws Exception{
		Collection<Widget> col = parse("test_resources/input/jsp_directive.txt");
		Widget widget = (Widget)col.toArray()[0];
		Widget widget1 = (Widget)col.toArray()[1];
		assertEquals(2, col.size());
		assertEquals("jsp_directive", widget.getType());
		assertEquals("jsp_directive", widget1.getType());
		assertEquals(" page contentType=\"text/html;charset=ISO-8859-1\" language=\"java\"", widget.getText());
		assertEquals(" taglib uri=\"/WEB-INF/lib/widgets_allianz.jar\" prefix=\"widget\"", widget1.getText());
	}	
	
	
	@Test
	public void parser_When_parse_jsp_script() throws Exception{
		String lineSeparator = System.getProperty("line.separator");
		Collection<Widget> col = parse("test_resources/input/jsp_script.txt");
		assertEquals(1, col.size());
		Widget widget = (Widget)col.toArray()[0];
		assertEquals("jsp_script", widget.getType());
		String text = "String applicationTitle = (String)request.getSession().getAttribute(\"applicationTitle\");"+
		" BeanList menu = (BeanList) SEAUtils.getData(request, \"menu\");"+
		" String actionURL = request.getContextPath().concat(\"/busquedaLog/control.do\");"+
		" LogBean logBean = (LogBean)SEAUtils.getData(request, \"logBean\");";
		assertEquals(text, widget.getText());
	}
	
	@Test
	public void parser_When_parse_html_comment() throws Exception{
		Collection<Widget> col = parse("test_resources/input/html_comment.txt");
		assertEquals(1, col.size());
		Widget widget = (Widget)col.toArray()[0];
		assertEquals("html_comment", widget.getType());
	}
	
	@Test
	public void parser_When_parse_section_with_html_comment() throws Exception{
		Collection<Widget> col = parse("test_resources/input/section_with_html_comment.txt");
		assertEquals(1, col.size());
		Widget widget = (Widget)col.toArray()[0];
		assertEquals("section", widget.getType());
		Collection<Widget> children = widget.getChildren();
		assertEquals(5, children.size());
		Widget child1 = (Widget)children.toArray()[0];
		assertEquals("html_comment", child1.getType());
	}	
	
	
	
	@Test
	public void parser_When_parse_complete_page() throws Exception{
		Collection<Widget> col = parse("test_resources/input/page.txt");
		assertEquals(4, col.size());
		Widget widget1 = (Widget)col.toArray()[0];
		Widget widget2 = (Widget)col.toArray()[1];
		Widget widget3 = (Widget)col.toArray()[2];
		Widget widget4 = (Widget)col.toArray()[3];
		assertEquals("jsp_directive", widget1.getType());
		assertEquals("jsp_directive", widget2.getType());
		assertEquals("jsp_directive", widget3.getType());
		assertEquals("page", widget4.getType());
		Collection<Widget> children = widget4.getChildren();
		assertEquals(2, children.size());
		Widget child1 = (Widget)children.toArray()[0];
		Widget child2 = (Widget)children.toArray()[1];
		assertEquals("errors", child1.getType());
		assertEquals("appPage", child2.getType());
		//...
		//NOTA: Realmente este test no aporta nada,
		//      nunca ha petado así que siguiendo TDD
		//      sería un test inútil. Lo dejo más
		//      como test "completo" de una página.
	}




	
	
	//--------------------------------------------------------------------------------------------------------------------------------------------------- 
	
	
	private void parser_input_text(String file) throws Exception {
		Collection<Widget> col = parse(file);
		Widget wText = (Widget)col.toArray()[0];
		assertEquals("inputText", wText.getType());
		Collection<Attribute> attributes = wText.getAttributes();
		assertAttribute(attributes,"id","buscar");
		assertAttribute(attributes,"label","label.buscar");
		assertAttribute(attributes,"beanName","buscarBean");
		assertAttribute(attributes,"beanProperty","texto");
	}	
	
	private static Collection<Widget> parse(String file)throws Exception{
		String text = FileUtils.readFileToString(new File(file));
		WidgetParser wp = new WidgetParser(text);
		Collection<Widget> col = wp.parse();	
		return col;
	}
	
	private static Attribute searchAttribute(String name, Collection<Attribute> attributes){
		Attribute at = null;
		Attribute pointer = null;
		boolean found = false;
		Iterator<Attribute> it = attributes.iterator();
		while ( (it.hasNext()) && (!found) ) {
			pointer = it.next();
			if ( name.equals(pointer.getName())  ){
				at = pointer;
				found = true;
			}
		}
		return at;
	}
	
	private void assertAttribute(Collection<Attribute> attributes, String name, String value) {
		Attribute at = searchAttribute(name, attributes);
		assertEquals(name, at.getName());
		assertEquals(value, at.getValue());
	}	
	
	
	
	

}
