/*******************************************************************************
 * WidgetTranslatorTest.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.parser.widgets;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import org.apache.commons.io.FileUtils;
import org.junit.Test;


public class WidgetTranslatorTest {
	
	@Test
	public void translate_simple_widget_page_to_prime_faces_mobile()throws Exception{
		String actual = translate("test_resources/input/simple_widget_page.txt","templates/primefaces_mobile");
		assertEquals(true, actual.contains("<f:view xmlns=\"http://www.w3.org/1999/xhtml\""));
		assertEquals(true, actual.contains("<pm:page title=\"\">"));
		assertEquals(true, actual.contains("</f:view>"));
	}
	
	@Test
	public void translate_general_widget_page_assert_form_to_prime_faces_mobile()throws Exception{
		String actual = translate("test_resources/input/general_widget_page.txt","templates/primefaces_mobile");
		assertEquals(true, actual.contains("id=\"formMenu\""));
		assertEquals(true, actual.contains("id=\"mainForm\""));
	}
	
	
	@Test
	public void translate_simple_widget_section_to_prime_faces_and_multi_language_label()throws Exception{
		String actual = translate("test_resources/input/section.txt","templates/primefaces");
		assertEquals(true, actual.contains("<p:tab title=\"#{msg['label.listin\']}\" id=\"sectionPaginaCompleta\">"));
	}
	
	
	@Test
	public void translate_widget_sectionButton_to_prime_faces()throws Exception{
		String actual = translate("test_resources/input/sectionButton.txt","templates/primefaces");
		assertEquals(true, actual.contains("<p:commandButton value=\"#{msg['label.buscar']}\" action=\"/command/main.com.listin.commands.BuscarCommand.xhtml\" />"));
	}
	
	@Test
	public void translate_widget_inputText_to_prime_faces()throws Exception{
		String actual = translate("test_resources/input/inputtext.txt","templates/primefaces");
		assertEquals(true, actual.contains("<p:inputText value=\"#{buscarBean.texto}\" id=\"buscar\" type=\"text\"/>"));
	}
	
	@Test
	public void translate_widget_table_to_prime_faces()throws Exception{
		String actual = translate("test_resources/input/table.txt","templates/primefaces");
		assertEquals(true, actual.contains("<p:dataTable var=\"item\" value=\"#{personaBeanList}\" id=\"personaBeanList\" >"));
	}
	
	@Test
	public void translate_widget_tableColumn_to_prime_faces()throws Exception{
		String actual = translate("test_resources/input/tableColumn.txt","templates/primefaces");
		assertEquals(true, actual.contains("<h:outputText value=\"#{item.idPersona}\" />"));
	}		
	
	
	
	
	//@Test
	//public void translate_complete_example_alta()throws Exception{
	//	String actual = translate("test_resources/input/alta.jsp","templates/primefaces");
	//	System.out.println(actual);
	//}
	
	
	
	@Test
	public void getEvent_Test()throws Exception{
		String seaEvent = "sendSEAEvent(mainForm,'buscar')";
		assertEquals("buscar",WidgetTranslator.getEvent(seaEvent));
	}


	
	
	//-----------------------------------------------------------------------------------------------
	
	private String translate(String file, String templates) throws FileNotFoundException, IOException {
		Collection<Widget> widgets = parseWidgetPage(file);
		File templateDir = new File(templates);
		Collection<EventView> eventsView = new ArrayList<EventView>();
		eventsView.add(new EventView("buscar", "/command/main.com.listin.commands.BuscarCommand.xhtml"));
		WidgetTranslator trans = new WidgetTranslator(widgets,templateDir,eventsView);
		String actual = trans.translate();
		return actual;
	}	

	private Collection<Widget> parseWidgetPage(String file) throws FileNotFoundException, IOException {
		FileInputStream fis = new FileInputStream(file);
		WidgetPage wPage = new WidgetPage(fis);
		wPage.parsePage();
		return wPage.getWidgets();
	}	
	

}
