/*******************************************************************************
 * MigratorPopUpStart.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.migrator.sea.jsf.popup.actions;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.migrator.sea.jsf.Activator;
import com.migrator.sea.jsf.MigratorDataBean;
import com.migrator.sea.jsf.wizards.MigratorWizard;

public class MigratorPopUpStart implements IObjectActionDelegate {

	private Shell shell;
	
	/**
	 * Constructor for Action1.
	 */
	public MigratorPopUpStart() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		//IProject selectedProject = getSelectedIProject();
		IProject selectedProject = null;
		   IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		    if (window != null)
		    {
		        IStructuredSelection selection = (IStructuredSelection) window.getSelectionService().getSelection();
		        Object firstElement = selection.getFirstElement();
		        if (firstElement instanceof IAdaptable)
		        {
		            selectedProject = (IProject)((IAdaptable)firstElement).getAdapter(IProject.class);
		            //IPath path = project.getFullPath();
		            //System.out.println(path);
		        }
		    }		
		
		try{
			if (isSeaProject(selectedProject)){
				Shell shell = new Shell();
				MigratorWizard wizard = new MigratorWizard();
				MigratorDataBean migratorDataBean =  createMigratorDataBean(selectedProject);
				System.out.println(migratorDataBean.getSeaProjectDirectory());
				wizard.setMigratorDataBean(migratorDataBean);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.create();
				dialog.open();			
			}
			else{
				MessageDialog.openInformation(shell, "JsfMigrator", "Nothing to do. The selected project is not a project developed with SEA.");			
			}			
		}catch (Exception e){
			e.printStackTrace();
		}
	}
	
	
	private MigratorDataBean createMigratorDataBean(IProject selectedProject)throws IOException {
		String seaProjectName = selectedProject.getName();
		String jsfProjectName = "";
		if (seaProjectName.contains("Sea")){
			jsfProjectName = seaProjectName.replaceFirst("Sea", "Jsf");
		}
		else if (seaProjectName.contains("Sea")){
			jsfProjectName = seaProjectName.replaceFirst("SEA", "JSF");
		}
		else {
			jsfProjectName = jsfProjectName.concat("Jsf");
		}
		MigratorDataBean migratorDataBean = new MigratorDataBean();
		migratorDataBean.setSeaProjectName(seaProjectName);
		migratorDataBean.setSeaProjectDirectory(selectedProject.getLocation().toString());
		migratorDataBean.setOutputDirectory(ResourcesPlugin.getWorkspace().getRoot().getLocation().toString());
		migratorDataBean.setJsfProjectName(jsfProjectName);
		migratorDataBean.setPluginDirectory(getPluginDirectoryPath());
		return migratorDataBean;
	}
	
	
	private String getPluginDirectoryPath()throws IOException{
		URL url = getURLFromPluginFile(Activator.getDefault(), "");
		return FileLocator.toFileURL(url).getPath();
	}
	
    private URL getURLFromPluginFile (Plugin plugin, String resource) throws IOException {
    	if (plugin == null) throw new IllegalArgumentException("Expected plugin instance!");
        URL url = plugin.getBundle().getEntry(resource);
        if (url == null) throw new IllegalStateException("Expected "+resource+" into plugin!");
        return url;
    }	
	

	private boolean isSeaProject(IProject selectedProject) {
		IPath path = selectedProject.getLocation();
		File projectDirectory = path.toFile();
		return (new File(projectDirectory + "/.sea")).exists();
	}

//	protected IProject getSelectedIProject() {
//    	IProject selectedIProject = null;
//        ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getSelection();
//        if (selection instanceof IStructuredSelection) {
//           Object[] selectedObjects = ((IStructuredSelection)selection).toArray();
//           if (selectedObjects.length >= 0 && selectedObjects[0] instanceof IProject) {
//        	   selectedIProject = (IProject)selectedObjects[0];
//               return selectedIProject;
//           }
//        }
//        throw new IllegalStateException("IFile selected expected!");
//    }	
	
	

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

}
