/*******************************************************************************
 * MigratorWizard.java
 *  
 *  Copyright (c) 2013, Oliver Tena. All rights reserved.
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 * 
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301  USA
 ******************************************************************************/
package com.migrator.sea.jsf.wizards;

import java.io.File;
import java.io.IOException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Shell;
import com.migrator.sea.jsf.MigratorDataBean;
import com.seatojsf.appflowmig.Project;

public class MigratorWizard extends Wizard {
	
	
	private MigratorWizardPage wizardPage;
	private MigratorDataBean migratorDataBean;
	private Shell shell;
	
	

	@Override
	public boolean performFinish() {
		boolean canFinish = true;;
		wizardPage.refreshMigratorDataBean();
		try{
			if (validations()){
				canFinish = true;
				migrate();					
				MessageDialog.openInformation(shell, "JsfMigrator","Done! You can import the new JSF project in your workspace!");				
			}
			else{
				canFinish = false;
			}
		}catch (Exception e){
			e.printStackTrace();
		}
		return canFinish;
	}
	
	
	
	
	
	
	
	
	
	private void migrate() {
		String seaProjectPath = migratorDataBean.getSeaProjectDirectory();
		String seaSrcPath = seaProjectPath + "/JavaSource";
		System.out.println("seaSrcPath : " + seaSrcPath);
		File f = new File(seaSrcPath); 
		if (!f.exists()){
			seaSrcPath = seaProjectPath + "/src";
		}
		String seaWebContentPath = seaProjectPath + "/WebContent";
		String outputPath = migratorDataBean.getOutputDirectory();
		String projectName = migratorDataBean.getJsfProjectName();
		String configDir = migratorDataBean.getPluginDirectory() + "/configuration";
		String templateDir = migratorDataBean.getPluginDirectory() + "/templates/primefaces";
		
		System.out.println("seaProjectPath : " + seaProjectPath);
		System.out.println("seaSrcPath : " + seaSrcPath);
		System.out.println("seaWebContentPath : " + seaWebContentPath);
		System.out.println("outputPath : " + outputPath);
		System.out.println("projectName : " + projectName);
		System.out.println("configDir : " + configDir);
		System.out.println("templateDir : " + templateDir);
		
		Project project = new Project(new File(seaProjectPath), 
				new File(seaSrcPath), new File(seaWebContentPath), 
				new File(outputPath), projectName, new File(configDir), 
				new File(templateDir));
		project.migrate();
	}









	private boolean validations() throws IOException{
		boolean ok = true;;
		if (!(new File(migratorDataBean.getOutputDirectory())).exists()){
			wizardPage.setErrorMessage("It is mandatory that the output directory exist");
			ok = false;
		}
		String sJsfProjectDirectory = migratorDataBean.getOutputDirectory() + "/" + migratorDataBean.getJsfProjectName();
		File jsfProjectDirectory = new File(sJsfProjectDirectory);
		File seaProjectDirectory = new File(migratorDataBean.getSeaProjectDirectory());
		if (jsfProjectDirectory.getAbsoluteFile().getCanonicalFile().equals(seaProjectDirectory.getAbsoluteFile().getCanonicalFile())){
			wizardPage.setErrorMessage("Unable to create the new JSF project in the SEA project directory, please change output directory or JSF project name.");
			ok = false;
		}
		if (migratorDataBean.getJsfProjectName().trim().equals("")){
			wizardPage.setErrorMessage("JSF project name is a required field.");
			ok = false;
		}
		if (migratorDataBean.getOutputDirectory().trim().equals("")){
			wizardPage.setErrorMessage("Output directory is a required field.");
			ok = false;
		}		
		return ok;

	}









	public void addPages(){
		wizardPage = new MigratorWizardPage("Migrate project to JSF 2.0");
		wizardPage.setMigratorDataBean(migratorDataBean);
		addPage(wizardPage);
	}


	public void setMigratorDataBean(MigratorDataBean migratorDataBean) {
		this.migratorDataBean = migratorDataBean;
	}
	


}
